;;; -*- Mode: Lisp; Syntax: Common-Lisp; Package: CL-User; Base 10 -*-

;;; FILE:       Expermts/packages.lisp
;;; VERSION:    1.0
;;; PURPOSE:    Definition of the EXPMT package
;;; DEPENDS-ON: random.lisp
;;; PROGRAMMER: Alexander Alexandrov Petrov  (apetrov+@andrew.cmu.edu)
;;; CREATED:    22-03-99 [1.0]
;;; UPDATED:    16-04-99 [1.1]
;;; UPDATED:    30-08-00 [1.1.1]  Text shapes & text resources
;;; UPDATED:    ...


     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     ;;;;                                                       ;;;;
     ;;;;         E X P E R I M E N T   P A C K A G E S         ;;;;
     ;;;;                                                       ;;;;
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(cl:unless (cl:find-package "RANDOM")
  (cl:error "RANDOM package should be loaded prior to EXPMT package."))

(cl:defpackage  "EXPMT" (:use "COMMON-LISP" 
                              "CCL"       ; windows etc.
                              "RANDOM" )  ; see RANDOM.LISP
                        (:size 1000)
  (:export  ; from EXPERMTS/GENERAL.LISP
     "EXPERIMENT-OBJECT"
     "DUPLICATE"
     "INSTRUCTIONS-MIXIN"
     "START-INSTRUCT"
     "FINISH-INSTRUCT"

     "EXP-DEVICE"
     "EXP-INPUT-DEVICE"
     "EXP-OUTPUT-DEVICE"
     "EXP-I/O-DEVICE"
     "DUMMY-INPUT-DEVICE"
     "DUMMY-OUTPUT-DEVICE"
     "DUMMY-I/O-DEVICE"

     "STIMULUS"
     "REDRAWABLE-MIXIN"
     "REDRAWABLE-STIMULUS"
     "PRESENT"
     "PREPARE-PRESENTATION"
     "STIMULUS-CODE"
  )
  (:export  ; from EXPERMTS/HSV_COLOR.LISP
     "COLOR"
     "RGB-PICK-COLOR"
     "HSV-PICK-COLOR"
     "COLOR-HUE"
     "COLOR-SATURATION"
     "COLOR-VALUE"
     "COLOR-HSV-VALUES"
     "COLOR-RGB-VALUES"
  )
#|
  (:export  ; from EXPERMTS/CUT_PASTE.LISP
     "CUT-AND-PASTE-WINDOW"
     "CUT"
     "COPY"
     "PASTE"
     "CLEAR"
  )
|#
  (:export  ; from EXPERMTS/SHAPES.LISP
     "POINT"
     "ANGLE"
     "CANONIZE-ANGLE"
     "DRAW-OBJECT"
     "COLOR-MIXIN"
     "SHAPE"
     "COLOR-SHAPE"
     "SHAPE-ANCHOR-POINT"
     "SHAPE-COLOR"

     "DRAW"
     "SHAPE-WINDOW"
;;   "CUT-AND-PASTE-SHAPE-WINDOW"
     "WINDOW-SHAPES"

     "RECTANGLE"
     "COLOR-RECTANGLE"
     "MAKE-COLOR-RECTANGLE"
     "RECTANGLE-TOP-LEFT"
     "RECTANGLE-BOTTOM-RIGHT"

     "CIRCLE"
     "COLOR-CIRCLE"
     "MAKE-COLOR-CIRCLE"
     "CIRCLE-RADIUS"

     "TEXT-SHAPE"
     "COLOR-TEXT-SHAPE"
     "MAKE-TEXT-SHAPE"
     "MAKE-COLOR-TEXT-SHAPE"
     "TEXT-SHAPE-STRING"
     "TEXT-SHAPE-STREAM"

     "WEDGE"
     "COLOR-WEDGE"
     "MAKE-COLOR-WEDGE"
     "WEDGE-START-ANGLE"
     "WEDGE-SWEEP-ANGLE"

     "BASE-DAISY"
     "GENERAL-DAISY"
     "MAKE-GENERAL-DAISY"
     "GENERAL-DAISY-PETALS"
     "PETAL"
     "MAKE-PETAL"
     "PETAL-PARENT"

     "DAISY"
     "MAKE-DAISY"
     "DAISY-AXIS"
     "DAISY-PETAL-ANGLE"
     "DAISY-PETAL-COLORS"
  )
  (:export  ; from EXPERMTS/RT-METER.LISP
     "RT-METER"
     "RESET-RT-METER"
     "READ-RT-METER"
     "WAIT-FOR-KEY-PRESS"
     "IDLE-WAIT"
  )
  (:export  ; from EXPERMTS/INSTRUCT.LISP
     "INSTRUCTION-DIALOG"
     "INSTR-DIALOG-TEXTS"
     "INSTR-DIALOG-COMPLETION-HOOK"
     "INSTR-DIALOG-MODAL-P"
     "SHOW-INSTRUCTIONS"
     "WAIT-FOR-INSTRUCTIONS"
  )
  (:export  ; from EXPERMTS/TRIAL.LISP
     "TRIAL"
     "TRIAL-STATUS"
     "TRIAL-NUMBERS"
     "TRIAL-RESULT"
     "DO-TRIAL"
     "REPORT-TRIAL"
     "TRIAL-STATUS-CODE"

     "TRIAL-POOL"
     "TRIAL-POOL-SIZE"
     "TRIAL-POOL-RANDOMIZE-P"
     "TRIAL-POOL-REPEAT-ON-FAIL-P"
     "MAKE-TRIAL-POOL"
     "TRIAL-POOL-EMPTY-P"
     "NEXT-TRIAL"
     "PUSH-TRIAL"
     "ANY-TRIAL-FROM-POOL"
     "MERGE-TRIAL-POOLS"
     "RANDOMIZE-TRIAL-POOL"
  )
  (:export  ; from EXPERMTS/EXP_WIN.LISP
     "EXPERIMENT-WINDOW"
     "EXP-COMMENT"
     "EXP-SUBJECT-NUMBER"
     "EXP-DATA-FILE"
     "EXP-BLOCK-LIST"
     "EXP-CURRENT-BLOCK"
     "EXP-CUMUL-TRIAL-NO"
     "EXP-VISIBLE-STIMULI"
     "EXP-GRAPH-RESOURCES"
     "EXP-TEXT-RESOURCES"

     "SETUP-EXPERIMENT"
     "RUN-EXPERIMENT"
     "FINISH-EXPERIMENT"
     "TRIGGER-TRIAL"
     "CLEAR-WINDOW"

     "BLOCK"
     "BLOCK-PARENT"
     "BLOCK-TRIAL-POOL"
     "BLOCK-SUB-BLOCK-LIST"
     "BLOCK-CURRENT-SUB-BLOCK"
     "BLOCK-TRIAL-NO"
     "MAKE-BLOCK"
     "START-BLOCK"
     "FINISH-BLOCK"
     "RUN-BLOCK"

     "SUB-BLOCK"
     "SUB-BLOCK-PARENT"
     "SUB-BLOCK-CURRENT-TRIAL-NO"
     "SUB-BLOCK-CURRENT-TRIAL"
     "SUB-BLOCK-COMPLETED-TRIALS"
     "MAKE-SUB-BLOCK"
     "START-SUB-BLOCK"
     "RUN-SUB-BLOCK"
     "FINISH-SUB-BLOCK"
     "REPORT-SUB-BLOCK"

     "TRIGGER-TRIAL"
     "*TRIAL-TRIGGER-CHARS*"
  )
  ;; other exports ...
) ; defpackage EXPMT


;;;;;;;  End of file EXPERMTS/PACKAGES.LISP

;;; -*- Mode: Lisp; Syntax: Common-Lisp; Package: Expmt; Base: 10 -*-

;;; FILE:       Expermts/general.lisp
;;; VERSION:    1.0
;;; PURPOSE:    Definition of experiment-object, stimulus, and present
;;; DEPENDS-ON: packages.lisp
;;; PROGRAMMER: Alexander Alexandrov Petrov  (apetrov+@andrew.cmu.edu)
;;; CREATED:    22-03-99 [1.0]
;;; UPDATED:    ...

     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     ;;;;                                               ;;;;
     ;;;;         G E N E R A L   C L A S S E S         ;;;;
     ;;;;                                               ;;;;
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(in-package "EXPMT")


(defclass  experiment-object ()
  ()
  (:documentation "Top of the class hierarchy related to experiments." ))

(defgeneric duplicate (object &rest keyargs)
  (:documentation "Make a copy of OBJECT using fresh memory elements." ))

(defclass  instructions-mixin (experiment-object)
  ((start-instruct  :accessor  start-instruct
                    :type      (or string list pathname)
                    :initarg   :start-instruct
                    :initform  nil  )
   (finish-instruct :accessor  finish-instruct
                    :type      (or string list pathname)
                    :initarg   :finish-instruct
                    :initform  nil  )
  ))


(defclass exp-device (experiment-object)
  ()
  (:documentation "Window, speaker, ACT-R interface, etc." ))

(defclass exp-output-device (exp-device)
  () )
(defclass exp-input-device (exp-device)
  () )
(defclass exp-I/O-device (exp-input-device exp-output-device)
  () )

(defclass dummy-output-device (exp-output-device)
  () 
  (:documentation "Observes the output protocol but does nothing." ))
(defclass dummy-input-device (exp-input-device)
  ()
  (:documentation "Observes the input protocol but does nothing." ))
(defclass dummy-I/O-device (dummy-input-device dummy-output-device)
  ()
  (:documentation "Observes the I/O protocol but does nothing." ))


;;;; ---------------------------------------------------------------------------
;;;;
;;;; The STIMULUS class 
;;
;;   A base class for storing information about stimuli.  Each application
;;   must define subclasses of stimulus and appropriate methods for the
;;   generic function present (see below).
;;   A trial (see TRIAL.LISP) can contain a set of stimuli and present them
;;   simultaneously or successively.

(defclass  stimulus (experiment-object)
  ()
  (:documentation "Base class for stimuli presented during an experiment." ))

(defclass  redrawable-mixin (experiment-object)
  ((redraw-p  :reader     stimulus-redraw-p
              :allocation :class
              :initform   T  )
  )
  (:documentation "Mix-in for experiment-object that may need redrawing." ))

(defclass  redrawable-stimulus (stimulus redrawable-mixin)
  ()
  (:documentation "Stimulus that may need redrawing." ))


;;;; The PRESENT generic function
;;
;;   Presents a stimulus through some exp-output-device.
;;   Possible specifications of present include:
;;     -- draw a visual stimulus into a window
;;     -- play a sound
;;     -- fill in some slot of the ACT-R goal chunk

(defgeneric present (stimulus exp-output-device)
  (:documentation "Present a stimulus through some experiment output-device." ))

(defmethod  present ((stimulus stimulus) (dummy dummy-output-device))
  (declare (ignore stimulus dummy))
  nil )

(defgeneric prepare-presentation (stimulus exp-output-device &rest keyargs)
  (:documentation 
     "Prepare EXP-OUTPUT-DEVICE for one or more calls to PRESENT." ))
  ;; STIMULUS specifies the whole class of relevant stimuli.

(defmethod  prepare-presentation ((stimulus stimulus) 
                                  (dummy dummy-output-device)
                                  &rest keyargs)
  (declare (ignore stimulus dummy keyargs))
  nil )


;;;; The STIMULUS-CODE generic function
;;
;;  Generates a code that identifies the stimulus in the data file.
;;  Codes are usually sequences (i.e. lists) of numbers.

(defgeneric stimulus-code (stimulus)
  (:documentation "Code identifying the stimulus in the data file." ))


;;;;;;;  End of file EXPERMTS/GENERAL.LISP

;;; -*- Mode: Lisp; Syntax: Common-Lisp; Package: Expmt; Base: 10 -*-

;;; FILE:       Expermts/HSV_color.lisp
;;; VERSION:    1.0
;;; PURPOSE:    Hue-saturation-value (HSV) color space
;;; DEPENDS-ON: packages.lisp
;;; PROGRAMMER: Alexander Alexandrov Petrov  (apetrov+@andrew.cmu.edu)
;;; CREATED:    20-11-98 [1.0]
;;; UPDATED:    16-04-99 Integrated with the EXPMT package
;;; UPDATED:    ...

     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     ;;;;                                               ;;;;
     ;;;;      HUE-SATURATION-VALUE  COLOR  SPACE       ;;;;
     ;;;;                                               ;;;;
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(in-package "EXPMT")

;; This file defines the functions RGB-PICK-COLOR
;; and HSV-PICK-COLOR that imitate the color picker.
;; (Compare with USER-PICK-COLOR, p.255 of the manual.)
;;
;; It also defines the functions COLOR-HUE, COLOR-SATURATION,
;; COLOR-BRIGHTNESS, and COLOR-HSV-VALUES.
;;
;; The arguments to these functions are percentages
;; (0-100) and degrees (0-359) rather than bytes.
;;
;; Examples:
;;    (RGB-pick-color 100   0   0) --> 16711680
;;    (HSV-pick-color   0 100 100) --> 16711680  ; RED has hue=000
;;    (color-values 16711680)      -->  65535  0  0
;;    (RGB-pick-color   0 100   0) -->    65280
;;    (HSV-pick-color 120 100 100) -->    65280  ; GREEN has hue=120
;;    (color-values 65280)         -->  0  65535  0
;;    (RGB-pick-color   0   0 100) -->      255
;;    (HSV-pick-color 240 100 100) -->      255  ; BLUE has hue=240
;;    (color-values 255)           -->  0  0  65535
;;    (RGB-pick-color  50  50  50) -->  8421504  ; 50% gray
;;    (HSV-pick-color any   0  50) -->  8421504
;;    (color-values 83557110)      -->  32896  32896  32896
;;    (RGB-pick-color  50   0   0) -->  8388608
;;    (HSV-pick-color   0 100  50) -->  8323072
;;    (color-values 8323072)       -->  32896  0  0


;;;;;;;;;;  COLOR data type
;;

(deftype color ()
  'integer )   ; 3x8=24 bits representation


;; The following functions and macros are documented
;; in the MCL manual (Chapter 6, pp. 249-256):
#|

*color-available*                               [Variable]
*black-color*                                   [Variable]
 ... white, pink, red, orange, yellow, green,
 ... dark-green, light-blue, blue, purple, brown,
 ... tan, light-gray, gray,  
*dark-gray-color*                               [Variable]
*black-RGB*                                     [Variable]
*white-RGB*                                     [Variable]

(make-color red green blue)  -->  color         [Function]
(color-values color)  -->  (values r g b)       [Function]
(color-red color)     -->  red                  [Function]
(color-green color)   -->  green                [Function]
(color-blue color)    -->  blue                 [Function]
(real-color-equal color1 color2)  --> boolean   [Function]

(color-to-RGB color &optional RGB-macptr) --> RGB-record    [Function]
  ;; Ex: (print-record (color-to-RGB *white-color*) :RGBcolor)) 
  ;;     #<Record :RGBCOLOR :RED 65535 :GREEN 65535 :BLUE 65535>
(RGB-to-color RGB-record)  -->  color           [Function]
(with-RGB (variable color) {form}* )            [Macro]

(user-pick-color &key :color :prompt :position) --> color  [Function]

|#


;; What is not documented is that QuickDraw supports other color
;; formats as well. In particular, there is the HSV (hue-saturation-value)
;; format, which approximates the Munsell system.
;; There are two traps -- RGB2HSV and HSV2RGB -- that convert between
;; the two color formats.


;;;;;;;;;;;;;  New function definitions
;;
;;  These imitate certain submenus of the color picker.
;;  Compare with USER-PICK-COLOR.


(defun RGB-pick-color (red green blue)
  "Each argument varies from 0 to 100 and is interpreted as percent."
  (declare (type fixnum red green blue))
  (assert (and (<= 0 red   100)
               (<= 0 green 100)
               (<= 0 blue  100)))
  (flet ((percent->16bit (percent)
           (round (* #xFFFF (/ percent 100))) ))
    (make-color (percent->16bit red)
                (percent->16bit green)
                (percent->16bit blue)) ))

;; HUE is between 0 and 359.
;; SATURATION and VALUE are between 0 and 100.
;; Returns a color (i.e. an integer).
(defun HSV-pick-color (hue saturation value)
  "Hue varies from 0 to 359, Saturation and Value vary from 0 to 100."
  (declare (type fixnum hue saturation value))
  (assert (and (<= 0 hue        359)
               (<= 0 saturation 100)
               (<= 0 value      100)))
  (let ((hue-16bit        (round (* (/ #xFFFF 360) hue)))
        (saturation-16bit (round (* (/ #xFFFF 100) saturation)))
        (value-16bit      (round (* (/ #xFFFF 100) value))) )
    (rlet  ((hsv :HSVcolor :hue hue-16bit
                           :saturation saturation-16bit
                           :value value-16bit)
            (rgb :RGBcolor) )
 ;;   (print-record hsv :HSVcolor) (terpri)
    (#_HSV2RGB hsv rgb)        ; convert HSV to RGB
 ;;   (print-record rgb :RGBcolor) (terpri)
    (RGB-to-color rgb) )))     ; convert RGB to color


;;;;;;;
(defun color-hue (color)
  "Returns an angle from 0 to 359. R=0, G=120, B=240."
  (with-rgb (rgb color)
    (rlet ((hsv :HSVcolor))
      (#_RGB2HSV rgb hsv)
      (let ((16bit (rref hsv :HSVcolor.hue)))
        (values (round (* 360 (/ 16bit #xFFFF)))) ))))

(defun color-saturation (color)
  "Returns a value from 0 to 100."
  (with-rgb (rgb color)
    (rlet ((hsv :HSVcolor))
      (#_RGB2HSV rgb hsv)
      (let ((16bit (rref hsv :HSVcolor.saturation)))
        (values (round (* 100 (/ 16bit #xFFFF)))) ))))

(defun color-value (color)
  "Black=0, White=100"
  (with-rgb (rgb color)
    (rlet ((hsv :HSVcolor))
      (#_RGB2HSV rgb hsv)
      (let ((16bit (rref hsv :HSVcolor.value)))
        (values (round (* 100 (/ 16bit #xFFFF)))) ))))

(defun color-HSV-values (color)
  "Returns three values: hue (0-359), saturation (0-100), and value (0-100)."
  (with-rgb (rgb color)
    (rlet ((hsv :HSVcolor))
      (#_RGB2HSV rgb hsv)
      (let ((hue-16bit        (rref hsv :HSVcolor.hue))
            (saturation-16bit (rref hsv :HSVcolor.saturation))
            (value-16bit      (rref hsv :HSVcolor.value)) )
        (values (round (* 360 (/ hue-16bit        #xFFFF)))
                (round (* 100 (/ saturation-16bit #xFFFF)))
                (round (* 100 (/ value-16bit      #xFFFF)))) ))))

(defun color-RGB-values (color)
  "Returns three values: red (0-100), green (0-100), and blue (0-100)."
  (flet ((16bit->percent (16bit)
           (round (* 100 (/ 16bit #xFFFF))) ))
    (multiple-value-bind (r g b) (color-values color)
      (values (16bit->percent r)
              (16bit->percent g)
              (16bit->percent b)) )))


;;;;;;;  End of file EXPERMTS/HSV_COLOR.LISP

;;; -*- Mode: Lisp; Syntax: Common-Lisp; Package: Expmt; Base: 10 -*-

;;; FILE:       Expermts/shapes.lisp
;;; VERSION:    1.0
;;; PURPOSE:    Drawing simple shapes (e.g. circle, square).
;;; DEPENDS-ON: packages.lisp, HSV_color.lisp, general.lisp, cut_paste.lisp
;;; PROGRAMMER: Alexander Alexandrov Petrov  (apetrov+@andrew.cmu.edu)
;;; CREATED:    20-11-99 [1.0]
;;; UPDATED:    ...

       ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
       ;;;;                                             ;;;;
       ;;;;         D R A W I N G   S H A P E S         ;;;;
       ;;;;                                             ;;;;
       ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(in-package "EXPMT")


;; This file defines the class SHAPE and its subclasses.
;; It also defines the class SHAPE-WINDOW which is suitable
;; for displaying shapes.
;; The generic function DRAW is defined along with methods 
;; for certain shapes.

;;;;;;;;;;  Points
;;

(deftype point ()
  'integer )   ; use QuickDraw's internal representation

;; See the functions MAKE-POINT, SUBTRACT-POINT, etc. in
;; Chapter 2 of the MCL manual.  
;; The read macro #@ converts a subsequent list of two integers
;; into a point.


;;;;;;;;;;  Angles
;;

(deftype angle ()
  'integer )

(defun canonize-angle (any-angle)
  "Bring into the interval [-360,+360]."
  (if (<= -360 any-angle 360)
      any-angle
      (mod any-angle 360) ))


;;;;;;;;;;  Base shape classes
;;

(defclass  draw-object  (experiment-object)  ; see EXPERMTS/GENERAL.LISP
  ()
  )  ; draw-object


(defclass  shape (draw-object)
  ((anchor-point  :accessor   shape-anchor-point
                  :initarg    :anchor-point
                  :type       point
                  :initform   #@(0 0)  )
  ))  ; shape

(defclass  color-mixin (draw-object)
  ((color         :accessor   shape-color
                  :initarg    :color
                  :type       color
                  :initform   *black-color*  )   ; defined by MCL
  ))  ; color-mixin

(defclass  color-shape (shape color-mixin)
  () )  ; color-shape


;;;;;;;;;;  Generic function for drawing shapes
;;
;;  Make sure to embed each call to DRAW into
;;  a WITH-FOCUSED-VIEW form!

(defgeneric draw (shape)
  (:method (unknown-shape)
     (cerror "Don't draw anything and continue."
             "DRAW has no method for shapes of type ~S."
             (type-of unknown-shape)) )
  (:documentation "Draws SHAPE in the current port, possibly with color." ))

(defmethod draw :around ((color-shape color-shape))
  (with-fore-color (shape-color color-shape)
     (call-next-method) ))


;;;;;;;;;;;  Shape windows
;;
;;  Windows that keep a list of shapes and redraw them each time
;;  the window needs refreshing.

(defclass shape-window (exp-output-device window)  ;; or COLOR-DIALOG
  ((shape-list  :accessor window-shapes
                :initarg  :shapes
                :type     list     ; of shapes
                :initform nil )
  )
  (:default-initargs 
     :window-type   :document-with-grow    ;; or :SINGLE-EDGE-BOX
     :window-title  "Shape window" )
  )  ; shape-window

(defmethod view-draw-contents ((win shape-window))
  ;; Specialize this system-supplied generic function
  ;; so that shapes are re-drawn upon necessity.
  (with-focused-view win
    (dolist (shape (window-shapes win))
      (draw shape) )))

(defmethod (setf window-shapes) :after (new-value (win shape-window))
  (declare (ignore new-value))
  (invalidate-view win T) )   ; redraw everything (if visible)


;;;;;;;;;;;  Cut-and-paste shape windows
;;
#|  ; Depends on CUT_PASTE.LISP, which is not included in ONE_FILE.LISP
(defclass cut-and-paste-shape-window  (shape-window cut-and-paste-window)
  ()
 )
|#

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;  Rectangles
;;

(defclass  rectangle  (shape)
  ((anchor-point :accessor   rectangle-top-left  )
   (bottom-right :accessor   rectangle-bottom-right
                 :type       point
                 :initarg    :bottom-right
                 :initform   #@(10 10)  )
  ))  ; rectangle

(defclass  color-rectangle  (rectangle color-shape)
  () )  ; color-rectangle

(defun make-color-rectangle  (top-left bottom-right color)
  (declare (type point top-left bottom-right)
           (type color color) )
  (make-instance 'color-rectangle
                 :anchor-point top-left
                 :bottom-right bottom-right
                 :color        color  ))

(defmethod  draw ((rectangle rectangle))
  (rlet ((rect :rect 
               :topLeft  (rectangle-top-left rectangle)
               :botRight (rectangle-bottom-right rectangle)))
    (#_PaintRect rect) ))


;;;;;;;;;  Circles
;;

(defclass  circle  (shape)
  ((raduis        :accessor   circle-radius
                  :initarg    :radius
                  :type       integer
                  :initform   10  )
  ))  ; circle

(defclass  color-circle  (circle color-shape)
  () )  ; color-circle

(defun make-color-circle  (center radius color)
  (declare (type point center)
           (type integer radius)
           (type color color) )
  (make-instance 'color-circle
                 :anchor-point center
                 :radius       radius
                 :color        color  ))

(defmethod  draw ((circle circle))
  ;; Create an enclosing square and call #_PaintOval.
  (let ((center (shape-anchor-point circle))
        (offset (make-point (circle-radius circle)
                            (circle-radius circle))) )
    (rlet ((rect :rect 
                 :topLeft  (subtract-points center offset)
                 :botRight (add-points center offset)))
       (#_PaintOval rect) )))


;;;;;;;;;  Text shapes
;;

(defclass  text-shape  (shape)
  ((string        :accessor   text-shape-string
                  :initarg    :string
                  :type       string
                  :initform   "Dummy text..."  )
   (stream        :accessor   text-shape-stream
                  ;; Usually bound to a SHAPE-WINDOW
                  :initarg    :stream
                  :type       stream
                  :initform   *standard-output* )
  ))  ; text-shape

(defclass  color-text-shape  (text-shape color-shape)
  () )  ; color-text-shape

(defun make-text-shape  (pos string &optional (stream *standard-output*))
  (declare (type point pos)
           (type string string) )
  (make-instance 'text-shape
                 :anchor-point pos
                 :string  string
                 :stream  stream ))

(defun make-color-text-shape  (pos string color)
  (declare (type point pos)
           (type string string)
           (type color color) )
  (make-instance 'color-text-shape
                 :anchor-point pos
                 :string       string
                 :color        color  ))

(defmethod  draw ((text text-shape))
  ;; Move the pen to position and call PRINC.
  ;; It is assumed that the call to DRAW is enclosed in a
  ;; WITH-FOCUSED-VIEW form that focuses on the very window that
  ;; is accessible via the TEXT-SHAPE-STREAM slot.
  ;; Sorry for this mess. The class TEXT-SHAPE was defined post hoc.
  (let ((pos (shape-anchor-point text))
        (string (text-shape-string text)) )
    (#_MoveTo (point-h pos) (point-v pos))     ; in the focused view !!!
    (princ string (text-shape-stream text)) 
    string ))

;;; Example of text shapes in use
#|
(defparameter *text* (make-text-shape #@(100 100) "Hello world!"  ))
(defparameter *win* (make-instance 'shape-window :view-size #@(300 300) 
                                                 :view-position #@(300 200) ))
(setf (text-shape-stream *text*) *win*)    ;; DON'T FORGET THIS!!!
(set-back-color *win* *light-gray-color* )
(let ((font-descriptor (view-font *win*)))
  (setf (second font-descriptor) 30)       ; set font size to 30
  (set-view-font *win* font-descriptor) )
;; We are now ready to present the text in the window
(setf (window-shapes *win*) (list *text*))
|#


;;;;;;;;;  Wedges
;;

(defclass  wedge-mixin  (draw-object)
  ((start-angle  :accessor   wedge-start-angle
                 :initarg    :start-angle
                 :type       angle
                 :initform   0  )
   (sweep-angle  :accessor   wedge-sweep-angle
                 :initarg   :sweep-angle
                 :type      angle
                 :initform  360  )
  )
  (:documentation "Becomes a true WEDGE when merged with a circle." )
  )  ; wedge-mixin         ; also used in petals

(defclass  wedge  (circle wedge-mixin)
  () )  ; wedge

(defclass  color-wedge  (wedge color-circle)
  () )  ; color-wedge

(defun make-color-wedge  (center radius start-angle sweep-angle color)
  (declare (type point center)
           (type integer radius)
           (type angle start-angle sweep-angle)
           (type color color) )
  (make-instance 'color-wedge
                 :anchor-point center
                 :radius       radius
                 :start-angle  (canonize-angle start-angle)
                 :sweep-angle  (canonize-angle sweep-angle)
                 :color        color  ))

(defmethod  draw ((wedge wedge))
  ;; Create an enclosing square and call #_PaintArc.
  (let ((center (shape-anchor-point wedge))
        (offset (make-point (circle-radius wedge)
                            (circle-radius wedge))) )
    (rlet ((rect :rect 
                 :topLeft  (subtract-points center offset)
                 :botRight (add-points center offset)))
       (#_PaintArc rect 
                   (wedge-start-angle wedge)
                   (wedge-sweep-angle wedge)) )))


;;;;;;;;;  Base daisies and petals
;;

(defclass  base-daisy  (circle)
  () )  ; base-daisy

(defclass  petal  (wedge-mixin color-mixin)
  ((parent-daisy  :accessor  petal-parent
                  :type      (or null base-daisy)
                  :initarg   :parent
                  :initform  nil )
  ))  ; petal

(defun  make-petal (start-angle sweep-angle color)
  (declare (type angle start-angle sweep-angle)
           (type color) )
  (make-instance 'petal
                 :start-angle (canonize-angle start-angle)
                 :sweep-angle (canonize-angle sweep-angle)
                 :color       color  ))


(defclass  general-daisy  (base-daisy circle)
  ((petals  :accessor  general-daisy-petals
            :type      list   ; of petals
            :initarg   :petals
            :initform  nil )
  )
  (:documentation "Daisy with petals of arbitrary angles." )
  )  ; general-daisy

(defun  make-general-daisy (center radius petal-list)
  (declare (type point center)
           (type integer radius)
           (type list petal-list) )
  (let ((daisy (make-instance 'general-daisy
                              :anchor-point center
                              :radius       radius
                              :petals       petal-list)))
    (dolist (petal petal-list)
      (setf (petal-parent petal) daisy))
    daisy ))

(defmethod  draw ((daisy general-daisy))
  ;; Create an enclosing square and draw individual petals.
  (let ((center (shape-anchor-point daisy))
        (offset (make-point (circle-radius daisy)
                            (circle-radius daisy))) )
    (rlet ((rect :rect 
                 :topLeft  (subtract-points center offset)
                 :botRight (add-points center offset)))
      (dolist (petal (general-daisy-petals daisy))
        (with-fore-color (shape-color petal)
          (#_PaintArc rect 
                      (wedge-start-angle petal)
                      (wedge-sweep-angle petal)))) )))


;;;;;;;;;  Evenly spaced daisies
;;
;;  These are daisies with uniform petal-angles 
;;  and between-petal spaces.

(defvar  *default-daisy-radius* 100 )
(defvar  *default-daisy-axis* 0
  "Default orientation of first petal's midline." )
(defvar  *default-petal-angle* 60 )

(defclass  daisy  (base-daisy circle)
  ((axis          :accessor daisy-axis
                  :type     angle
                  :initarg  :axis
                  :initform *default-daisy-axis*
                  :documentation "Orientation of first petal's midline." )
   (petal-angle   :accessor daisy-petal-angle
                  :type     angle
                  :initarg  :petal-angle
                  :initform *default-petal-angle*  )
   (petal-colors  :accessor  daisy-petal-colors
                  :type      list   ; of colors
                  :initarg   :petal-colors
                  :initform  nil 
                  :documentation "Length implicitly defines the number of petals." )
   ;; Private slots (to optimize drawing).
   (top-left      :accessor  daisy-top-left     ; of the enclosing rectangle
                  :type      point
                  :initform  #@(0 0) )
   (bottom-right  :accessor  daisy-bottom-right ; of the enclosing rectangle
                  :type      point
                  :initform  #@(0 0) )
   (start-angle   :accessor  daisy-start-angle  ; for first call to #_PaintArc
                  :type      angle
                  :initform  0 )
   (delta-angle   :accessor  daisy-delta-angle  ; from one petal to the next
                  :type      angle              ; negative for counter-clockwise
                  :initform  0 )
  )
  (:documentation "Daisy with evenly spaced petals." )
  )  ; daisy

(defun  update-private-daisy-slots (daisy)
  ;; Called whenever some of the public slots is changed.
  (let ((center (shape-anchor-point daisy))
        (offset (make-point (circle-radius daisy)
                            (circle-radius daisy))) )
    (setf (daisy-top-left daisy) (subtract-points center offset))
    (setf (daisy-bottom-right daisy)  (add-points center offset)) )
  (let ((petal-angle (daisy-petal-angle daisy))
        (number-of-petals (length (daisy-petal-colors daisy))) )
    (setf (daisy-start-angle daisy)
          (canonize-angle (- (daisy-axis daisy)
                             (truncate petal-angle 2))))
    (cond ((zerop number-of-petals)            ; no petals at all
             (setf (daisy-delta-angle daisy) 0))
          ((plusp petal-angle)                 ; clockwise ?
             (setf (daisy-delta-angle daisy) (truncate +360 number-of-petals)))
          (t (setf (daisy-delta-angle daisy) (truncate -360 number-of-petals)))) )
  daisy )

(defmethod  (setf shape-anchor-point) :after (new-value (daisy daisy))
  (declare (ignore new-value))
  (update-private-daisy-slots daisy) )

(defmethod  (setf circle-radius) :after (new-value (daisy daisy))
  (declare (ignore new-value))
  (update-private-daisy-slots daisy) )

(defmethod  (setf daisy-axis) :after (new-value (daisy daisy))
  (declare (ignore new-value))
  (update-private-daisy-slots daisy) )

(defmethod  (setf daisy-petal-angle) :after (new-value (daisy daisy))
  (declare (ignore new-value))
  (update-private-daisy-slots daisy) )

(defmethod  (setf daisy-petal-colors) :after (new-value (daisy daisy))
  (declare (ignore new-value))
  (update-private-daisy-slots daisy) )


(defun  make-daisy (center color-list &key (radius *default-daisy-radius*)
                                           (axis *default-daisy-axis*)
                                           (petal-angle *default-petal-angle*) )
  "Make a daisy with evenly spaced petals of specified colors."
  (declare (type point center)
           (type integer radius)
           (type angle axis petal-angle)
           (type list color-list) )
  (update-private-daisy-slots (make-instance 'daisy
                                :anchor-point center
                                :radius       radius
                                :axis         (canonize-angle axis)
                                :petal-angle  (canonize-angle petal-angle)
                                :petal-colors color-list) ))

(defmethod  draw ((daisy daisy))
  (let ((curr-angle  (daisy-start-angle daisy))
        (delta-angle (daisy-delta-angle daisy))
        (sweep-angle (daisy-petal-angle daisy)) )
    (rlet ((rect :rect 
                 :topLeft  (daisy-top-left daisy)
                 :botRight (daisy-bottom-right daisy)))
      (dolist (color (daisy-petal-colors daisy))
        (with-fore-color color
          (#_PaintArc rect curr-angle sweep-angle))
        (incf curr-angle delta-angle)) )))


;;;;;;;;;;  Example
;;
#|
(defparameter *daisy* (make-daisy #@(150 150)
                                  (list *yellow-color*   *orange-color*
                                        *pink-color*     *red-color*
                                        *brown-color*    *purple-color*
                                        *blue-color*     *green-color*  )
                                  :petal-angle 30 ) )

(defparameter *win* (make-instance 'shape-window :shapes (list *daisy*)
                                                 :view-size #@(300 300) 
                                                 :view-position #@(300 200) ))
(set-back-color *win* *light-gray-color* )

(dotimes (i (length (daisy-petal-colors *daisy*)) :done)
  (message-dialog "  Loves me, doesn't love me..."
                  :ok-text "Go" )
  (pop (daisy-petal-colors *daisy*))
  (invalidate-view *win* T) )
|#

;;;;;;;  End of file EXPERMTS/SHAPES.LISP

;;;  -*- Mode: LISP; Package: EXPMT; Syntax: COMMON-LISP;  Base: 10 -*-

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Author      : Alexander Petrov
;;;
;;; Address     : Department of Psychology
;;;             : Carnegie Mellon University
;;;             : 5000 Forbes Ave             
;;;             : Pittsburgh,PA 15213-3890
;;;             : apetrov+@andrew.cmu.edu
;;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Filename    : RT_meter.lisp
;;; Version     : 1.1
;;; Depends on  : packages.lisp, general.lisp
;;; Created     : 27-11-98 [1.0]
;;; Updated     : 15-04-98 [1.1]  Support for maximal wait times.
;;; Updated     : 22-04-01 [1.2]  Use of *EVENTHOOK*.
;;; 
;;; Description : Provides RT-METER -- a mix-in class for measuring reaction
;;;             : times.  Subject responses are key presses on the ordinary
;;;             : keyboard.  Uses the general-purpose event handlers of MacOS to
;;;             : trap the key presses and the function get-internal-real-time
;;;             : to measure time.
;;;
;;; Bugs        : Doesn't scale the times using the system-supplied constant 
;;;             : internal-time-units-per-second.  Simply assumes that its 
;;;             : value is 1000 (as it is in MCL 4.2).  Thus, if this constant
;;;             : changes in the future, all timers will get off the mark.
;;; 
;;; To do       : increase robustness by judicious use of without-interrupts
;;; 
;;; 
;;; ----- History -----
;;; 27.11.98 Alexander Petrov -- version 1.0
;;; 15.04.99 Alexander Petrov -- version 1.1 -- Support for maximal wait times.
;;; 22.04.01 Alexander Petrov -- version 1.2 -- Use of *EVENTHOOK*.
;;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(in-package "EXPMT")


;;;; ---------------------------------------------------------------------------
;;;;
;;;; Event hook
;;
;;   The function RT-METER-EVENT-HOOK gets attached to the system variable
;;   *EVENTHOOK*  (see methods READ-FLAG-AND-ENABLE and READ-FLAG-AND-DISABLE
;;   below). See also the :before method for VIEW-KEY-EVENT-HANDLER.
;;
;;   The variable *EVENTHOOK* provides a user hook into the event system.
;;   If this variable contains a function, the function is called and has the
;;   first chance of processing any events.  The eventhook function must take
;;   no arguments.  It can access the global variable *CURRENT-EVENT*.  If the
;;   eventhook function returns T, the processing of that event is considered
;;   complete.  If the eventhook function returns NIL, the system-supplied event
;;   handlers are invoked.

(defvar  *time-of-last-KEYDWNEVT*  0
  "Set by RT-METER-EVENT-HOOK" )

(defun  RT-meter-event-hook ()
"For KeyDwnEvt, record the time stamp of its occurence in *TIME-OF-LAST-KEWDWNEVT*
and process it it the usual way; Ignore the events of all other types."
  (cond ((eq $KeyDwnEvt           ; when the *current-event* is a KeyDown event
            (rref *current-event* :eventrecord.what))
         (setq *time-of-last-KEYDWNEVT* (get-internal-real-time))
         (setq *eventhook* nil)   ; stop meddling with the system event handlers
         nil )          ; give VIEW-KEY-EVENT-HANDLER a chance to run
        (t  nil) ))     ; let events of all other types go through

(defun  install-RT-METER-EVENT-HOOK ()
  (setq *time-of-last-KEYDWNEVT* 0)    ; just in case
  (setq *eventhook* #'RT-meter-event-hook) 
  T)


;;;; ---------------------------------------------------------------------------
;;;;
;;;; The RT-meter class 
;;
;;   A mix-in class for measuring reaction times.  
;;   ...
;;   The RT-meter sould be the front window when calling get-response.


(defclass  RT-meter (exp-input-device window)
  ((char       :accessor  RT-meter-char
               :type      (or character (member nil :expired))
               :initform  nil  )
   (start-time :accessor  RT-meter-start-time
               :type      integer     ; as returned by get-internal-real-time
               :initform  0  )
   (end-time   :accessor  RT-meter-end-time
               :type      integer
               :initform  0  )
   (enabled-p  :reader    RT-meter-enabled-p
               :initform  nil  )
  )
  (:documentation "Mix-in class for measuring reaction times." ))


(defgeneric read-flag-and-enable (RT-meter)
  (:method ((RT-meter RT-meter))
     (without-interrupts
       (let ((previous-state (RT-meter-enabled-p RT-meter)))
         (when (null previous-state)
           (setf (slot-value RT-meter 'enabled-p) T)
           (install-RT-METER-EVENT-HOOK) )
         previous-state )))
  (:documentation
"Set RT-meter to listen to key presses. This involves setting RT-METER-EVENT-HOOK.
Return previous mode." ))

(defgeneric  read-flag-and-disable (RT-meter)
  (:method ((RT-meter RT-meter))
     (setq *eventhook* nil)      ; stop meddling with the system event handlers
     (without-interrupts
       (let ((previous-state (RT-meter-enabled-p RT-meter)))
         (when previous-state
           (setf (slot-value RT-meter 'enabled-p) nil))
         previous-state )))
  (:documentation
    "Set RT-meter not to listen to key presses. Return previous mode." ))



;;;;;;;;;  Keystroke trap
;;
;;  See Chapter 10 in MCL Reference Manual (and in particular the section
;;  on the built-in event handlers).
;;
;;  Bug:  If two keystrokes come in very rapid succession, the second one
;;        will override the first. This happens whenever the second key
;;        interrupts before the first one has managed to disable the RT-meter.
;;        There is no easy way out of this. Wrapping the entire method in
;;        without-interrupts does not eliminate the problem because the
;;        time interval spent on generic dispatch is still interruptable.

(defgeneric record-key-press (RT-meter char time)
  (:method  ((RT-meter RT-meter) (char character) (time integer))
    ;; (read-flag-and-disable RT-meter)     ; just in case
    (setf (RT-meter-char RT-meter) char)
    (setf (RT-meter-end-time RT-meter) time) )
  (:documentation "Called on keyboard interrupts inside enabled RT-meters." ))

(defmethod view-key-event-handler :before ((RT-meter RT-meter) 
                                               (char character) )
  (when (read-flag-and-disable RT-meter)  ; was enabled?
    ; subsequent key presses are now disabled; *EVENTHOOK* is restored to NIL.
    ; RT-EVENT-HOOK has stored the time in *TIME-OF-LAST-KEYDWNEVT*.
    (record-key-press RT-meter char *time-of-last-KEYDWNEVT*) ))


;;;;;;;;;;  User-level functions
;;
;;

(defgeneric reset-RT-meter (RT-meter &optional start-time)
  (:documentation "Reset the timer and activate the RT-meter." ))

(defmethod reset-RT-meter ((RT-meter RT-meter) &optional start-time)
  (read-flag-and-disable RT-meter)            ; ignore keypress interrupts
  (let ((curr-time (get-internal-real-time)))
    (setf (RT-meter-char RT-meter) nil)       ; undefined char
    (if (integerp start-time)                 ; explicitly supplied
        (setf (RT-meter-start-time RT-meter) start-time)
        (setf (RT-meter-start-time RT-meter) curr-time ))
    (setf (RT-meter-end-time   RT-meter) 0)   ; return negative RTs if broken
    (window-select RT-meter)                  ; bring window to front
    (read-flag-and-enable RT-meter)           ; listen to keypress interrupts
    (RT-meter-start-time RT-meter) ))


(defgeneric read-RT-meter (RT-meter)
  (:method  ((RT-meter RT-meter))
     (values (RT-meter-char RT-meter)
             (- (RT-meter-end-time RT-meter)
                (RT-meter-start-time RT-meter))) )
  (:documentation "Returns two values: the last key pressed and the RT." ))


(defgeneric wait-for-key-press (RT-meter &optional max-wait-time)
  (:method  ((RT-meter RT-meter) &optional (max-wait-time nil))
     (when (RT-meter-enabled-p RT-meter)
       (if (null max-wait-time)            ; no deadline
           (do ()
               ((RT-meter-char RT-meter))  ; until the subject presses some key
             ) ; busy wait
           (let ((deadline (+ max-wait-time   ; in msec
                              (RT-meter-start-time RT-meter))) )
             (do ()
                 ((RT-meter-char RT-meter))   ; until the S presses some key OR
               (when (> (get-internal-real-time) deadline)
                 (read-flag-and-disable RT-meter)  ; the gate closes, sorry...
                 (setf (RT-meter-char RT-meter) :expired)
                 (setf (RT-meter-end-time RT-meter) deadline)
                 (return) )                   ; terminate the loop
               )))) ; busy wait
     (read-RT-meter RT-meter) )
  (:documentation 
"Ensures that a key-press event has occurred and then reads RT-meter. If 
MAX-WAIT-TIME (in msec) is specified and no key-press occurs, return :EXPIRED" ))


;;;;;;;;  Examples
#|
Example1:
  (defclass my-class (other-stuff RT-meter)
    ... )

  (defvar *my-window* (make-instance 'my-class))

  (defun do-trial (my-window trial-descriptor)
    ...
    (display-stimuli my-window trial-descriptor)
    (reset-RT-meter my-window)
    (multiple-value-bind (char RT)
                         (wait-for-key-press my-window)
      (if (member char '(#\l #\L #\r #\R))
          (write-data-file char RT)
          (warning "Press L or R") ))
    (clear-stimuli my-window)
    ... )


Example2:
  (defun do-self-terminating-trial (my-window trial-descriptor
                                    &optional (max-wait-time 5000) )
    ...
    (reset-RT-meter my-window)
    (start-playing-long-sound trial-descriptor)
    (multiple-value-bind (char RT)
                         (wait-for-key-press my-window max-wait-time)
      (case char
        ((#\L #\l)  (write-data-file "left"  RT))
        ((#\R #\r)  (write-data-file "right" RT))
        (:expired   (write-data-file "no key pressed" RT))
        (T          (warning "Press L or R")) ))
    ... )

  (defmethod record-key-press :after ((my-window RT-meter) char time)
    (declare (ignore char time))
    (stop-playing-any-sounds ))  ; terminate sound on early responses
|#




;;;;;;;;;;;;;;;  Additional RT-related functions  ;;;;;;;;;;;;
;;

(defun idle-wait (delay &optional (mode :duration))
  "Sits there, spinning, doing nothing, for the specified delay (in ms)"
  (let* ((internal (/ (* delay 1000) internal-time-units-per-second))
         (deadline (ecase mode
                     (:duration (+ internal (get-internal-real-time)))
                     (:deadline internal))) )
    (do ()
        ((> (get-internal-real-time) deadline) delay) )))


;;;;;;;;;
;;
;;  The functions below poll the keyboard directly and
;;  get accurate reaction time measures.  Temporal resolution
;;  on the order of 5 msec.
;;  (c) Mike Byrne, 1998
#|

(defconstant *keymap*  (make-record :keymap) )
(defconstant *map-ptr* (rref *keymap* :keymap.array) )

(defun wait-for-spacebar (&optional (start (get-internal-real-time)))
  "Return reaction time in milliseconds."
  (without-interrupts
    (#_GetKeys *keymap*)
    (do ()
        ((test-for-spacebar-down))
      (#_GetKeys *keymap*))
    (- (get-internal-real-time) start) ))


(defun test-for-spacebar-down ()
  (if (= 2 (%get-unsigned-byte *map-ptr* 6))
    t
    nil))
|#

;;;;  From Frank Lee
;;;;
#| 
(loop
     (when (go? (get-macatc instruction)) (return t))
     (when (= i 30) (Check-Runways instruction))
     (ccl::do-event-queue event
       (when (eq $KeyDwnEvt (rref event :eventrecord.what))
         (activate-screen-update)))
     (when *update-screen*
       (progn
         (deactivate-screen-update)
         (event-dispatch)))
    ) 
|# 

;;; -*- Mode: Lisp; Syntax: Common-Lisp; Package: Expmt; Base: 10 -*-

;;; FILE:       Expermts/instruct.lisp
;;; VERSION:    1.0
;;; PURPOSE:    A simple class for presenting text instructions
;;; DEPENDS-ON: packages.lisp, general.lisp
;;; PROGRAMMER: Alexander Alexandrov Petrov  (apetrov+@andrew.cmu.edu)
;;;             Based on a program kindly provided by Mike Byrne.
;;; CREATED:    22-10-98 [1.0]
;;; UPDATED:    ...

     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     ;;;;                                                   ;;;;
     ;;;;       I N S T R U C T I O N   D I A L O G         ;;;;
     ;;;;                                                   ;;;;
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(in-package "EXPMT")


;;; The INSTRUCTION-DIALOG class
;;;
;;;   A simple class for presenting text instructions.  The way this class is
;;;   used is simple:  make an instance, and it will prompt for a text file
;;;   containing the instructions.  If that's not OK, you can just pass in the
;;;   text you want in the initarg.
;;;
;;;   A second issue is control--what to do when the instructions have been
;;;   presented.  Just pass whatever Lisp expression you want run as the
;;;   :completion initarg, and you'll be set.
;;;


(defclass instruction-dialog (exp-output-device dialog)
  ((instruction-texts :accessor instr-dialog-texts
                      :initarg  :texts
                      :initform "Instructions ..." )
   (completion-hook   :accessor instr-dialog-completion-hook
                      :initform nil
                      :initarg :completion )
   (modal-p           :accessor instr-dialog-modal-p 
                      :initform nil 
                      :initarg :modal-p )
   )
  (:default-initargs
    :window-type :movable-dialog
    :window-title "Instructions"
    :auto-position :CenterMainScreen
    :view-size #@(624 429)
    :close-box-p nil
    :view-subviews
    (list 
     (make-dialog-item  'default-button-dialog-item
                        #@(544 401) #@(66 18) "Continue"
                        #'(lambda (self) (continue-click (view-window self)))
                        :view-nick-name 'CONTINUE)
     (make-dialog-item  'static-text-dialog-item
                        #@(10 11) #@(595 373) ""
                        nil
                        :view-nick-name 'text
                        :view-font '("Geneva" 12 :SRCOR :PLAIN (:COLOR-INDEX 0)))
     (make-dialog-item  'static-text-dialog-item
                        #@(2 410) #@(433 15)
                        "Press the Return key when you are ready to go on."
                        nil
                        :view-font '("Geneva" 10 :SRCOR :BOLD (:COLOR-INDEX 0)))
    )))



;;; CONTINUE-CLICK      [Method]
;;; Description : When the 'Continue' button is clicked, pop the next block
;;;             : of text.  If there is no more text, then run the completion
;;;             : hook.

(defmethod continue-click ((self instruction-dialog))
  "Called when the 'Continue' button is clicked, advances the text or closes"
  (let ((the-text (pop (instr-dialog-texts self))))
    (if the-text
      (set-dialog-item-text (view-named 'TEXT self) the-text)
      (progn
        (when (not (instr-dialog-modal-p self)) 
          (window-close self))
        (eval (instr-dialog-completion-hook self))))))



;;; INITIALIZE-INSTANCE      [Method]
;;; Description : Puts the first text block into the window.

(defmethod initialize-instance :after ((self instruction-dialog) &rest keyargs) 
  "Pops the first instruction text into the window"
  (declare (ignore keyargs))
  (set-dialog-item-text (view-named 'TEXT self) 
                        (pop (instr-dialog-texts self))))


;;; WAIT-FOR-INSTRUCTIONS      [Function]
;;; Date        : 98.05.12
;;; Description : There are times when using the completion hook mechanism
;;;             : is less than optimal and a waiting function call is 
;;;             : preferred.  This function handles that need.

(defun wait-for-instructions (texts)
  "Displays <texts> as a modal dialog."
  (modal-dialog 
   (make-instance 'instruction-dialog :texts texts :modal-p t
                  :completion '(return-from-modal-dialog t))))

;;;;;;;;;;;;;;

(defgeneric show-instructions (instr &optional modal-p)
  (:documentation "Open an instruction dialog and display text in it." ))

(defmethod  show-instructions ((string string) &optional (modal-p T))
  (show-instructions (list string) modal-p) )

(defmethod  show-instructions ((stream stream) &optional (modal-p T))
  (with-open-file (str stream)
    (show-instructions (read str) modal-p)) )

(defmethod  show-instructions ((string-list list) &optional (modal-p T))
  (unless (null string-list)
    (if modal-p
      (wait-for-instructions string-list)
      (make-instance 'instruction-dialog
                     :texts string-list ))
    T ))



;;;;;;;;;;;;
#|  
(defun warning (string &rest keyargs)
  "Displays a warning dialog, with a cursor."
 ; (#_ShowCursor)
  (apply #'message-dialog string 
                          :position #@(150 150)
                          keyargs )
 ; (#_HideCursor) 
 )
|#

;;;;;;;  End of file EXPERMTS/INSTRUCT.LISP

;;; -*- Mode: Lisp; Syntax: Common-Lisp; Package: Expmt; Base: 10 -*-

;;; FILE:       Expermts/trial.lisp
;;; VERSION:    1.0
;;; PURPOSE:    Definition of trial, and trial-pool
;;; DEPENDS-ON: packages.lisp, general.lisp, random.lisp
;;; PROGRAMMER: Alexander Alexandrov Petrov  (apetrov+@andrew.cmu.edu)
;;; CREATED:    22-03-99 [1.0]
;;; UPDATED:    16-04-99 copy-trial subsumed by duplicate.
;;; UPDATED:    ...

     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     ;;;;                                         ;;;;
     ;;;;         TRIALS  AND  TRIAL  POOLS       ;;;;
     ;;;;                                         ;;;;
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(in-package "EXPMT")


;;;; ---------------------------------------------------------------------------
;;;;
;;;; The TRIAL class 
;;
;;   A base class for storing information about a tral.  Each application must
;;   define subclasses of trial containing slots for the appropriate stimuli.
;;   See EXPERIMTS/GENERAL.LISP for the base classes stimulus and exp-device.

(defclass trial (experiment-object)
  ((status    :accessor  trial-status
              :type      (member :waiting :current :completed :failed :repeat)
              :initarg   :status
              :initform  :waiting )
   (trial-nos :accessor  trial-numbers
              :type      list   ; (cumul-trialno block-no block-trialno ...)
              :initarg   :trial-numbers
              :initform  nil )
   (result    :accessor  trial-result
              :type      T      ; answers, RTs, etc.
              :initform  nil )
  )
  (:documentation "Base class describing trials during an experiment." ))


(defmethod duplicate ((trial trial) &key (status nil status-supplied-p)
                                          (trial-numbers nil numbers-supplied-p)
                                          (result nil result-supplied-p)
                                          &allow-other-keys )
  (let ((new-trial (make-instance (class-of trial))))
    (if status-supplied-p
        (setf (trial-status new-trial) status)
        (setf (trial-status new-trial) (trial-status trial)) )
    (if numbers-supplied-p
        (setf (trial-numbers new-trial) trial-numbers)
        (setf (trial-numbers new-trial) (trial-numbers trial)) )
    (if result-supplied-p
        (setf (trial-result new-trial) result)
        (setf (trial-result new-trial) (trial-result trial)) )
    new-trial ))
    ;;
    ;; Each subclass that defines additional slots must
    ;; provide :around methods for DUPLICATE to handle them.


;;;; The DO-TRIAL generic function
;;
;;   Runs a trial from start to finish. Must present stimuli to some output
;;   device, then collect data from some input device, store the data
;;   in the trial-result slot, and return a value. 
;;   The value returned should be non-NIL for successful trials and NIL for
;;   failed trials that should be re-run.
;;   Do-trial need not prepare or cleanup the devices but may do so.
;;

(defgeneric do-trial (trial exp-I/O-device)
  (:documentation "Present stimuli, record responses, return success-p." ))

(defmethod do-trial ((trial trial)
                     (dummy dummy-I/O-device) )
  (declare (ignore dummy))
  (setf (trial-status trial) :completed)        ; always succeed
  (setf (trial-result trial) "<dummy result>")
  T )


;;;; The REPORT-TRIAL generic function
;;
;;   Writes a text representation (for the data file) of the trial and its
;;   results to the given stream.  A typical method would write a line of 
;;   numbers suitable for feeding into statistical packages.
;;   The default method supplied here should be shadowed by more specific
;;   methods reporting the identity of the trial (e.g. by describing the
;;   stimuli).

(defgeneric report-trial (trial stream)
  (:documentation "Reports the trial and its results in a data file." ))

(defmethod report-trial ((trial trial) (data-file stream))
  (format data-file "~& ~{~4D~} <unknown stimuli> ~D ~S~%"
                    (trial-numbers trial)
                    (trial-status-code (trial-status trial))
                    (trial-result trial)) 
  )  ; report-trial

(defun trial-status-code (status)
  (case status
    (:failed     0)
    (:completed  1)
    (:waiting    3)
    (:current    4)
    (:repeat     5)
    (t           9) ))


;;;; ---------------------------------------------------------------------------
;;;;
;;;; The TRIAL-POOL class 
;;
;;   A trial pool is a collection of trials waiting to be run.  Usually there 
;;   is one trial pool for each block of the experiment.  The trials are run 
;;   one by one until the pool is exhausted.  Unsuccessful trials may be 
;;   repeated.  The randomization may be off-line or on-line.

(defclass trial-pool (experiment-object)
  ((pool              :reader    trial-pool
                      :type      (simple-vector trial)
                      :initarg   :trial-pool  )
   (size              :accessor  trial-pool-size
                      :type      fixnum
                      :initarg   :size
                      :initform  0  )
   (randomize-p       :accessor  trial-pool-randomize-p
                      :type      (member T nil)
                      :initarg   :randomize-p
                      :initform  T  )
   (repeat-on-fail-p  :accessor  trial-pool-repeat-on-fail-p
                      :type      (member T nil)
                      :initarg   :repeat-on-fail-p
                      :initform  T  )
  )
  (:documentation "Pool of trials waiting to be run during a (sub)block." ))


(defun make-trial-pool (vector size &key (randomize-p T)
                                           (repeat-on-fail-p T rpt-supplied-p) )
  "Take a simple vector of trials and embed it into a TRIAL-POOL object."
  (declare (type simple-vector vector))
  (let ((repeat (if rpt-supplied-p repeat-on-fail-p randomize-p)))
    (make-instance 'trial-pool :trial-pool       vector
                               :size             size 
                               :randomize-p      randomize-p
                               :repeat-on-fail-p repeat )))

(defun trial-pool-empty-p (trial-pool)
  (not (plusp (trial-pool-size trial-pool))) )


(defgeneric  next-trial (trial-pool &optional randomize-p)
  (:documentation "Pick up (and remove) one of the trials from a pool." ))

;; Note that the method returns the *last* trial when RANDOMIZE-P is NIL.
;; Thus, to run trials in a pre-specified order, arrange them in reverse order
;; in the pool and avoid any calls to PUSH-TRIAL.
;;
(defmethod  next-trial ((pool trial-pool) 
                         &optional (randomize-p (trial-pool-randomize-p pool)) )
  (if (trial-pool-empty-p pool)
      nil
      (let* ((size   (trial-pool-size pool))
             (vector (trial-pool pool))
             (last-idx (- size 1))
             (index (if randomize-p
                        (random-fixnum size)  ; see RANDOM.LISP
                        last-idx ))
             (next-trial (svref vector index)) )
        (setf (svref vector index)            ; move last in the gap
              (svref vector last-idx))
        (decf (trial-pool-size pool))
        next-trial )))


(defgeneric  push-trial (trial trial-pool)
  (:documentation "Pushes TRIAL to TRIAL-POOL so that it can be re-run later." ))

(defmethod  push-trial ((trial trial) (pool trial-pool))
  ;; No overflow checking. The vector is not extendable, BEWARE!
  ;; Bug: When RANDOMIZE-P is NIL, the same trial will get stuck at the end of
  ;;      the vector and will be repeated over and over until it is :completed.
  (setf (svref (trial-pool pool) (trial-pool-size pool))
        trial)
  (incf (trial-pool-size pool))
  trial )

(defun any-trial-from-pool (trial-pool)
  "Gives some unspecified trial from a trial pool without removing the trial."
  ;; Useful for  (prepare-presentation trial exp-output-device)
  (if (trial-pool-empty-p trial-pool)
      nil
      (svref (trial-pool trial-pool) 0) ))

(defun merge-trial-pools (pool1 &rest more-pools)
  "Appends two or more trial pools (of the same type)."
  (declare (type trial-pool pool1))  ; and all the rest
  (let* ((all-pools (cons pool1 more-pools))
         (total-size (reduce #'+ all-pools :key #'trial-pool-size))
         (element-type (array-element-type (trial-pool pool1)))
         (new-vector (make-array total-size :element-type element-type))
         (index 0) )
    (dolist (pool all-pools)
      (let ((vector (trial-pool pool)))
        (dotimes (i (trial-pool-size pool))
          (setf (svref new-vector index) (svref vector i))
          (incf index) )))
    (assert (eq index total-size))
    (make-trial-pool new-vector total-size) ))


(defun randomize-trial-pool (trial-pool)
  "Assigns trials to random positions in the vector."
  (let ((vector (trial-pool trial-pool))
        (size   (trial-pool-size trial-pool)) )
    (flet ((swap (index1 index2)
             (let ((trial (svref vector index1)))
               (setf (svref vector index1) (svref vector index2))
               (setf (svref vector index2) trial) )))
      (dotimes (index size)
        (swap index (random-fixnum size)) )    ; see RANDOM.LISP
      trial-pool )))


(defmethod duplicate ((original trial-pool) 
                       &rest keyargs)    ; comply with the gfunction protocol
  (declare (ignore keyargs))
  (make-trial-pool (copy-seq (trial-pool original))
                   (trial-pool-size original)) )


;;;;;;;  End of file EXPERMTS/TRIAL.LISP

;;; -*- Mode: Lisp; Syntax: Common-Lisp; Package: Expmt; Base: 10 -*-

;;; FILE:       Expermts/trial.lisp
;;; VERSION:    1.1.1
;;; PURPOSE:    experiment-window, block, and sub-block classes
;;; DEPENDS-ON: packages.lisp, general.lisp, random.lisp
;;; PROGRAMMER: Alexander Alexandrov Petrov  (apetrov+@andrew.cmu.edu)
;;; CREATED:    22-03-99 [1.0]
;;; UPDATED:    16-04-99 [1.1]   Various parts moved to separate files.
;;; UPDATED:    30-08-00 [1.1.1] Add EXP-TEXT-RESOURCES slot.
;;; UPDATED:    ...

     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     ;;;;                                          ;;;;
     ;;;;    E X P E R I M E N T   W I N D O W     ;;;;
     ;;;;                                          ;;;;
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(in-package "EXPMT")



;;;; ---------------------------------------------------------------------------
;;;;
;;;; The EXPERIMENT-WINDOW class 
;;
;;   This is the main class of this library. An experiment window is the 'stage'
;;   for the experiment -- all stimuli are presented in it. Usually, its size is
;;   chosen to cover the whole screen, it has no title bar, and the cursor is
;;   hidden.
;;   The experiment window also traps keystrokes and measures reaction times.
;;   It inherits this functionality from the class RT-meter defined in 
;;   Expermts/RT_meter.lisp.
;;
;;   The experiment window is a hook for blocks, which in turn contain sub-blocks
;;   and trials. The generic funciton run-experiment is defined over experiment
;;   windows and is the main entry point to the experiment.
;;
;;   The class experiment-window has the following slots (in addition to the
;;   slots inherited from its various superclasses):
;;     * comment         -- a string or NIL
;;     * data-file       -- an output stream
;;     * subject-number  -- integer identifying the subject
;;     * block-list      -- a list of block objects (see below)
;;     * current-block   -- the block that is currently being run
;;     * cumul-trial-no  -- total number of trials since the beginning of
;;                            the experiment (across all blocks)
;;     * visible-stimuli -- a list of objects of type stimulus (see above)
;;                            These stimuli are presented when the window is
;;                            redrawn for whatever reason. Typically, do-trial
;;                            puts the current stimuli here, keeps them here
;;                            for a while, and then purges them out.  The
;;                            generic function invalidate-view is invoked on
;;                            the experiment window whenever the contents of
;;                            this slot is changed.  Thus, this slot is the main
;;                            'presentation screen' of the whole experiment.
;;     * graph-resources -- this slot supports graphical operations in the
;;                            experiment window by storing whatever resources
;;                            are necessary for the particular kind of stimuli:
;;                            points, rectangles, pens, etc.  Updated by the
;;                            generic function prepare-presentation on a 
;;                            block-by-block basis and used by present (see
;;     * text-resources  -- analogous to graph-resources
;;
;;   Any derived subclass must supply methods for the following generic funs:
;;     ...

(defclass experiment-window (exp-I/O-device instructions-mixin RT-meter  
                               color-dialog )   ;;; cut-and-paste-window
  ((comment         :accessor  exp-comment
                    :type      (or null string)
                    :initarg   :comment
                    :initform  nil  )
   (subject-number  :accessor  exp-subject-number
                    :type      integer
                    :initarg   :subject-number
                    :initform  000  )
   (data-file       :accessor  exp-data-file
                    :type      stream
                    :initarg   :data-file
                    :initform  (choose-new-file-dialog :prompt "Save data in:") )
   (block-list      :accessor  exp-block-list
                    :type      list
                    :initarg   :block-list
                    :initform  nil  )
   (current-block   :accessor  exp-current-block
                    :type      (or null integer)   ; index in EXP-BLOCK-LIST
                    :initarg   :current-block
                    :initform  nil  )
   (cumul-trial-no  :accessor  exp-cumul-trial-no
                    :type      integer
                    :initform  0  )
   (visible-stimuli :accessor  exp-visible-stimuli
                    :type      list
                    :initform  nil  )
   (graph-resources :accessor  exp-graph-resources
                    :type      T
                    :initform  nil  )
   (text-resources  :accessor  exp-text-resources
                    :type      T
                    :initform  nil  )
   )
  (:default-initargs
    :window-type :single-edge-box  ; i.e. no title bar, close-button, etc.
    :view-position (make-point 0 20)   ; 20 is the height of the menu bar
    :view-size     (make-point *screen-width* 
                               (- *screen-height* 20))
    :window-title  "Experiment window"
  ))  ;experiment-window


;;;;  Methods for drawing stimuli (see Expermts:shapes.lisp)

(defmethod (setf exp-visible-stimuli) :after (new-value (win experiment-window))
  (declare (ignore new-value))
  (invalidate-view win T) )   ; redraw everything (if visible)

(defmethod view-draw-contents ((win experiment-window))
  ;; This is a CCL function that is called whenever the view has to be redrawn.
  (with-focused-view win
    (dolist (shape (exp-visible-stimuli win))
      (draw shape) )))

(defgeneric clear-window (experiment-window)
  (:documentation "Removes all visible stimuli." )
  (:method ((exp-win experiment-window))
     (setf (exp-visible-stimuli exp-win) nil)) )


;;;; ---------------------------------------------------------------------------
;;;;
;;;; The BLOCK and SUB-BLOCK classes 
;;
;;   . . .

(defclass block (instructions-mixin experiment-object)
  ((parent          :accessor  block-parent
                    :type      (or null experiment-window)
                    :initarg   :parent  
                    :initform  nil  )
   (trial-pool      :accessor  block-trial-pool      ; all sub-blocks use
                    :type      (or null trial-pool)  ; the same trial-pool
                    :initarg   :trial-pool
                    :initform  nil  )
   (sub-block-list  :accessor  block-sub-block-list
                    :type      list
                    :initarg   :sub-block-list
                    :initform  nil  )
   (curr-sub-block  :accessor  block-current-sub-block
                    :type      (or null integer)     ; index in SUB-BLOCK-LIST
                    :initform  nil  )  
   (block-trial-no  :accessor  block-trial-no
                    :type      integer
                    :initform  0  )
  )
  (:documentation "A block of trials, organized in sub-blocks." ))


(defclass sub-block (instructions-mixin experiment-object)
  ((parent           :accessor  sub-block-parent
                     :type      block
                     :initarg   :parent  )
   (size             :accessor  sub-block-size
                     :type      (or integer (member :all))
                     :initarg   :size
                     :initform  0  )
   (curr-trial-no    :accessor  sub-block-current-trial-no
                     :type      integer
                     :initform  0  )
   (curr-trial       :accessor  sub-block-current-trial
                     :type      (or null trial)
                     :initform  nil  )
   (completed-trials :accessor  sub-block-completed-trials
                     :type      list
                     :initform  nil  )
  )
  (:documentation "A sub-block within a block of trials." ))


(defun make-block (trial-pool sub-block-list 
                              &key parent start-instruct finish-instruct)
  (declare (type trial-pool trial-pool)
           (type list sub-block-list) )
  (let ((block (make-instance 'block :trial-pool trial-pool
                                     :sub-block-list sub-block-list
                                     :parent parent
                                     :start-instruct  start-instruct
                                     :finish-instruct finish-instruct )))
    (dolist (sub-block sub-block-list)
      (setf (sub-block-parent sub-block) block) )
    block ))


(defun make-sub-block (size &key parent start-instruct finish-instruct)
  (declare (type (or integer (member :all)) size)
           (type block parent) )
  (make-instance 'sub-block :size size
                            :parent parent
                            :start-instruct  start-instruct
                            :finish-instruct finish-instruct ))


;;;;;;;;;

(defgeneric start-sub-block  (sub-block) )
(defgeneric run-sub-block    (sub-block) )  ; <-- main entry point
(defgeneric finish-sub-block (sub-block) )
(defgeneric report-sub-block (sub-block) )

(defmethod start-sub-block ((sub-block sub-block))
  (show-instructions (start-instruct sub-block))
  (setf (sub-block-current-trial-no sub-block) 0)
  (setf (sub-block-current-trial    sub-block) nil)
  (setf (sub-block-completed-trials sub-block) nil) )

(defmethod finish-sub-block ((sub-block sub-block))
  (show-instructions (finish-instruct sub-block)) )

(defmethod  run-sub-block ((sub-block sub-block))
  (cond ((null (sub-block-parent sub-block))
            0 )    ; do nothing and return # trials run
        (t  (start-sub-block sub-block)
            (run-sub-block-aux sub-block)    ; <-- the main work
            (report-sub-block  sub-block)    ; <-- 
            (finish-sub-block sub-block)
            (sub-block-current-trial-no sub-block) )))  ; return # trials run

(defun run-sub-block-aux (sub-block)
  (let* ((parent-block (sub-block-parent sub-block))
         (parent-window (block-parent parent-block))
         (trial-pool (block-trial-pool parent-block)) 
         (representative-trial (any-trial-from-pool trial-pool))
         (repeat-on-fail-p (trial-pool-repeat-on-fail-p trial-pool)) )
    (labels ((trial-counters ()   ; increment automatically
               (list (+ 1 (exp-current-block parent-window))
                     (+ 1 (block-current-sub-block parent-block))
                     (incf (exp-cumul-trial-no parent-window))
                     (incf (block-trial-no parent-block))
                     (incf (sub-block-current-trial-no sub-block)) ))
             (do-next-trial ()
                (let ((trial (next-trial trial-pool)))
                  (setf (trial-numbers trial) (trial-counters))
                  (setf (sub-block-current-trial sub-block) trial)
                  (setf (trial-status trial) :current)
                  (if (do-trial trial parent-window)
                      (setf (trial-status trial) :completed) ; OK
                      (setf (trial-status trial) :failed) )  ; timeout,bad key...
                  (when (and repeat-on-fail-p
                             (eq (trial-status trial) :failed))
                      (push-trial (duplicate trial :result nil 
                                                   :status :repeat)
                                  trial-pool))
                  (push trial (sub-block-completed-trials sub-block)) )))
      ;; Main body of RUN-SUB-BLOCK-AUX
      (unless (null representative-trial)
        (prepare-presentation representative-trial parent-window) )
      (if (eq :all (sub-block-size sub-block))
          (do ()
              ((trial-pool-empty-p trial-pool))     ; until all gone
            (do-next-trial))
          (dotimes (no (sub-block-size sub-block))  ; fixed sub-block size
            (when (trial-pool-empty-p trial-pool)   ; not enough trials ?
              (return))                             ; finish prematurely
            (do-next-trial)) )
      (setf (sub-block-completed-trials sub-block)
            (nreverse (sub-block-completed-trials sub-block))) )))


(defmethod  report-sub-block ((sub-block sub-block))
  (let* ((parent-block (sub-block-parent sub-block))
         (parent-experiment (block-parent parent-block))
         (subject-number (exp-subject-number parent-experiment))
         (data-file (exp-data-file parent-experiment)) )
    (with-open-file (stream data-file
                            :direction :output :if-exists :append)
      (dolist (trial (sub-block-completed-trials sub-block))
        (format stream "~%~3D " subject-number)
        (report-trial trial stream))) ))


;;;;;;;;;;;;;;;;;;;;;

(defgeneric start-block  (block) )
(defgeneric run-block    (block &optional start-sub-block-no) )
(defgeneric finish-block (block) )

(defmethod start-block ((block block))
  (show-instructions (start-instruct block))
  (setf (block-current-sub-block block) 0)
  (setf (block-trial-no block) 0) )

(defmethod finish-block ((block block))
  (show-instructions (finish-instruct block)) )

(defmethod run-block ((block block) &optional (start-sub-block-no 0))
  (start-block block)
  (let ((sub-blocks (block-sub-block-list block)))
    (do ((sb-no start-sub-block-no (+ 1 sb-no)))
        ((>= sb-no (length sub-blocks)))
      (setf (block-current-sub-block block) sb-no)
      (run-sub-block (nth sb-no sub-blocks)) ))
  (finish-block block) )



;;;;;;;;;;;;;;;;;;;;;

(defgeneric setup-experiment (experiment-window block-list subject-number 
                                                           &rest keyargs )
  (:documentation "Prepare an experiment for running." ))

(defgeneric run-experiment (experiment-window)
  (:documentation "Run all the blocks associated with the experiment." ))

(defgeneric finish-experiment (experiment-window)
  (:documentation "Close everything and lock the data file." ))


(defmethod  setup-experiment ((exp-win experiment-window) 
                                (block-list list)
                                subject-number
                                &key comment  start-instruct  finish-instruct
                                     (data-file nil data-file-supplied-p)
                                     (current-block 0)
                                     (cumul-trial-no 0) 
                                     (background-color *white-color*)
                                     (foreground-color *black-color*)
                                     &allow-other-keys )
  (setf (exp-block-list exp-win) block-list)
  (dolist (block block-list)
    (setf (block-parent block) exp-win))
  (setf (exp-subject-number exp-win) subject-number)
  (setf (exp-comment exp-win) comment)
  (setf (start-instruct  exp-win) start-instruct)
  (setf (finish-instruct exp-win) finish-instruct)
  (set-back-color exp-win background-color)
  (set-fore-color exp-win foreground-color)
  (when data-file-supplied-p
    (setf (exp-data-file exp-win) data-file))
  (with-open-file (data-file (exp-data-file exp-win)
                             :direction :output 
                             :if-exists :append
                             :if-does-not-exist :create)
    (format data-file "Data file ...~%" ) )
  (setf (exp-current-block  exp-win) current-block)
  (setf (exp-cumul-trial-no exp-win) cumul-trial-no)
  exp-win )


(defmethod run-experiment ((exp-win experiment-window))
  (window-select exp-win)    ; bring window on top of all windows
  (show-instructions (start-instruct exp-win))
  (seed-random-state)       ; re-set the random generator, see Random.lisp
  (do ()
      ((>= (exp-current-block exp-win)
           (length (exp-block-list exp-win))))
    (run-block (nth (exp-current-block exp-win)
                    (exp-block-list exp-win)))
    (incf (exp-current-block exp-win)) )
  (finish-experiment exp-win) 
  (exp-data-file exp-win) )


(defmethod finish-experiment ((exp-win experiment-window))
  "At the end, do all this"
  (beep)
  (set-mac-file-creator (exp-data-file exp-win) :|R*ch|)
 (lock-file (exp-data-file exp-win))
  (window-close exp-win)
  (show-instructions (finish-instruct exp-win) NIL) )  ; non-modal dialog


;;;;;;;;  Trial trigger utility
;;

(defvar *trial-trigger-chars* '(#\Space))

(defun trigger-trial (prompts exp-win 
                         &optional (trigger-chars *trial-trigger-chars*))
  "Draw the prompts in the window and wait for triggering keystroke"
  (declare (type shape prompt))             ; see Expermts:shapes.lisp
  (setf (exp-visible-stimuli exp-win) prompts)
  (loop
    (reset-RT-meter exp-win)                ; start listening to keyboard
    (multiple-value-bind (char RT)          ; see Expermts:RT_meter.lisp
                         (wait-for-key-press exp-win)
      (when (find char trigger-chars)
        (return (values char RT)))) ))


;;;;;;;  End of file EXPERMTS/EXP_WIN.LISP

;;;  -*- Mode: LISP; Package: CL-USER; Syntax: COMMON-LISP;  Base: 10 -*-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Fdbk_exp1.lisp
;;
;; April 12, 2001    (40th anniversary of Gagarin's flight)
;;
;; Depends on: HSV_color.lisp, shapes.lisp, RT_meter.lisp, exp_win.lisp
;; Note: Fdbk_exp1 does NOT depend on CatRat1/schedule.lisp

(in-package :cl-user)

(require "RANDOM")
(use-package "RANDOM")

;;(require "EXPMT")      ; see Programs/Expermts/trial.lisp
(use-package "EXPMT")


;;;;;;;;;  Global variables
;;

(defparameter *dot-radius*   8 
  "'Dots' are actually small circles." )

(defparameter *background-color* *black-color* )
(defparameter *foreground-color* *white-color* )

(defparameter *min-X* 61)
(defparameter *max-X* (+ 961 1))
(defparameter *min-Y* 193)
(defparameter *max-Y* (+ 593 1))


;;;;;;;;;  Stimulus sets
;;

(defun make-number-sequence (min max &optional (step 1))
  "A list of numbers: {min, min+step, min+2*step, ..., max}"
  (cond ((minusp step) 
            (nreverse (make-number-sequence max min (- step))) )
        ((zerop step)
            (warn "Cannot make any progress when STEP equals zero.") )
        (t  (do* ((result nil)
                  (curr-elt min (+ curr-elt step)) )
                 ((> curr-elt max) (nreverse result))
              (push curr-elt result)) )))

(defparameter *7-categ-1-stim-each-40-pix-apart*
  ;; {(category-member-list category-label)}+
  '(((420) 1)
    ((460) 2)
    ((500) 3)
    ((540) 4)
    ((580) 5)
    ((620) 6)
    ((660) 7) ))


;;;;;;;;;  Schedule bookkeeping
;;
;;  Compare with CatRat1/general.lisp which is NOT used here.

(defun schedule->code (schedule)
  "Maps keywords (e.g. :uniform) to integers."
  ;; See also MAKE-MINI-TRIAL-POOL below
  (ecase schedule
    ((0 :init)  0)        ; initial   (uniform with feedback)
    ((1 :U-fb)  1)        ; uniform with feedback
    ((2 :U-no)  2)        ; uniform w/o  feedback
    ((3 :P-fb)  3)        ; positive skew with feedback
    ((4 :P-no)  4)        ; positive skew w/o  feedback
    ((5 :N-fb)  5)        ; negative skew with feedback
    ((6 :N-no)  6) ))     ; negative skew w/o  feedback

#|
(defun code->schedule (code)
  "Maps integers to keywords (e.g. :uniform)."
  (ecase code
    ((0 :init)  :init)
    ((1 :U-fb)  :U-fb)
    ((2 :U-no)  :U-no)
    ((3 :P-fb)  :P-fb)
    ((4 :P-no)  :P-no)
    ((5 :N-fb)  :N-fb)
    ((6 :N-no)  :N-no) ))
|#

;;;;;;;;;  Stimulus class
;;
;;  Based on the class stimulus in Expermts/general.lisp.

(defclass  dot-pair-stimulus (stimulus)
  ((distance :accessor stimulus-distance
             :initarg  :distance
             :initform 0
             :type     fixnum )
   (category :accessor stimulus-category
             :initarg  :category
             :initform 0
             :type     fixnum )
   (X1       :accessor stimulus-X1
             :initarg  :X1
             :initform 0
             :type     fixnum )
   (X2       :accessor stimulus-X2
             :initarg  :X2
             :initform 0
             :type     fixnum )
   (Y       :accessor stimulus-Y
             :initarg  :Y
             :initform 0
             :type     fixnum )
  )
  (:documentation "Two white dots forming a horizontal segment." ))

(defun  make-dot-pair-stimulus (distance category
                                   &key (X1 :randomized) (Y :randomized))
  (declare (type fixnum distance category)
           (type (or fixnum (member :randomized)) X1 Y) ) 
  (when (eq Y :randomized)
    (setq Y  (random-fixnum-range *min-Y* *max-Y*)))   ; see Random.lisp
  (when (eq X1 :randomized)
    (setq X1 (random-fixnum-range *min-X* (- *max-X* distance))))
  (make-instance 'dot-pair-stimulus
                 :distance distance
                 :category category
                 :X1       X1
                 :X2       (+ X1 distance)
                 :Y        Y  ))

(defmethod stimulus-code ((dot-pair dot-pair-stimulus))
  (list (stimulus-distance dot-pair)
        (stimulus-category dot-pair)
        (stimulus-X1 dot-pair) 
        (stimulus-X2 dot-pair) 
        (stimulus-Y  dot-pair) ))


;;;;;;;;;  Feedback support

(defconstant *default-text-stimulus-anchor-point*
             (make-point 300 300) )

(defclass  text-stimulus (stimulus)
  ((string     :accessor  text-stimulus-string
               :initarg   :string
               :initform  "?"  )
   (point      :accessor  text-stimulus-point    ; SW corner of the text box
               :initarg   :point
               :initform  *default-text-stimulus-anchor-point*)
  )
  (:documentation "Trial presenting text on the screen." ))

(defun  make-text-stimulus (string-or-number  
                                   &key (point *default-text-stimulus-anchor-point*))
  (declare (type (or string number) string-or-number))
  (make-instance 'text-stimulus
                 :string  (princ-to-string string-or-number)
                 :point   point) )

;; Cache a vector of text trials, one for each digit.
;; 0 means 'no feedback' and presents a capital X on the screen.
(let* ((max-feedback 7)   ; hardwired max, change if necessary
       (feedback-objects (map 'simple-array
                              #'make-text-stimulus
                              (make-number-sequence 0 max-feedback))) )
  (setf (text-stimulus-string (svref feedback-objects 0)) "X")    ; noninformative fdbk
  (defun  corresp-feedback-object (feedback)
    (if (and (integerp feedback) (<= 0 feedback max-feedback))
        (svref feedback-objects feedback)
        (make-text-stimulus feedback)) ))


;;;;;;;;;  Trial classes
;;
;;  Based on the classes trial and stimulus in Expermts/exp_win.lisp.

(defclass  dot-pair-trial (trial dot-pair-stimulus)
  ((feedback   :accessor  dot-trial-feedback
               :initarg   :feedback
               :type      integer     ; passed to CORRESP-FEEDBACK-OBJECT
               :initform  0  )
   (schedule   :accessor  dot-trial-schedule
               :type      fixnum      ; see schedule->code
               :initarg   :schedule
               :initform  99  )
   (time-stamp :accessor  dot-trial-time-stamp
               :type      fixnum
               :initform  -9999  )
  )
  (:documentation "Trial presenting two white dots on a horizontal segment." ))


(defun  make-dot-pair-trial (distance category feedback
                                &key (schedule 99)
                                     (X1 :randomized) (Y :randomized)
                                     (trial-numbers '(0 0 0 0 0))
                                     (status :waiting) )
  (declare (type fixnum distance category)
           (type (or fixnum (member :category)) feedback)  ; Note: NOT a text-stimulus obj
           (type (or fixnum (member :randomized)) X1 Y)
           (type (member :waiting :current :completed :failed) status)
           (type list trial-numbers) )      ; (total-no block-no ...)
  (when (eq Y :randomized)
    (setq Y  (random-fixnum-range *min-Y* *max-Y*)))   ; see Random.lisp
  (when (eq X1 :randomized)
    (setq X1  (random-fixnum-range *min-X* (- *max-X* distance))))
  (make-instance 'dot-pair-trial :distance distance
                      :category category
                      :feedback (if (eq feedback :category) category feedback)
                      :schedule schedule
                      :X1 X1  :X2 (+ X1 distance)
                      :Y  Y   :status status
                      :trial-numbers trial-numbers ))


(defmethod duplicate :around ((original dot-pair-trial)
                               &key distance  category  feedback 
                                    X1 X2 Y   schedule  time-stamp
                               &allow-other-keys )
  (let ((new-object (call-next-method)))   ; call primary method
    (if distance
        (setf (stimulus-distance new-object) distance)
        (setf (stimulus-distance new-object) (stimulus-distance original)) )
    (if category
        (setf (stimulus-category new-object) category)
        (setf (stimulus-category new-object) (stimulus-category original)) )
    (if X1
        (setf (stimulus-X1 new-object) X1)
        (setf (stimulus-X1 new-object) (stimulus-X1 original)) )
    (if X2
        (setf (stimulus-X2 new-object) X2)
        (setf (stimulus-X2 new-object) (stimulus-X2 original)) )
    (if Y
        (setf (stimulus-Y new-object) Y)
        (setf (stimulus-Y new-object) (stimulus-Y original)) )
    (if feedback
        (setf (dot-trial-feedback new-object) feedback)
        (setf (dot-trial-feedback new-object) (dot-trial-feedback original)) )
    (if schedule
        (setf (dot-trial-schedule new-object) schedule)
        (setf (dot-trial-schedule new-object) (dot-trial-schedule original)) )
    (if time-stamp
        (setf (dot-trial-time-stamp new-object) time-stamp)
        (setf (dot-trial-time-stamp new-object) (dot-trial-time-stamp original)) )
    new-object ))


;;;;;;  The feedback is presented at the position of the stimulus

(defun  feedback-position (dot-pair-trial)
  (let ((X1 (stimulus-X1 dot-pair-trial))
        (X2 (stimulus-X2 dot-pair-trial))
        (Y  (stimulus-Y  dot-pair-trial)) )
    (make-point (- (floor (+ X1 X2) 2) 50)    ; to the left of the midpoint
                (+ Y  50)) ))                 ; below the line


;;;;;;;;;;;;;;;;  Presenting stimuli in the main window

(defun center-point (win)
  "Returns the center of WIN as a point."
  (let* ((size   (view-size win))
         (width  (point-h size))
         (height (point-v size)) )
    (make-point (truncate width  2)
                (truncate height 2)) ))

(defmethod prepare-presentation ((dot-trial dot-pair-trial)
                                    (exp-win experiment-window) 
                                    &key (radius *dot-radius*)
                                         (color  *foreground-color*) )
  (declare (ignore dot-trial))   ; the instance stands for its class
  (let ((dot1 (make-color-circle #@(100 300) radius color))
        (dot2 (make-color-circle #@(800 300) radius color))
        (text (make-text-shape (center-point exp-win)  ; see shapes.lisp
                               "?"                     ; string
                               exp-win)) )             ; stream
    (setf (exp-graph-resources exp-win)
          (list dot1 dot2))             ; dot-pair-resource
    (setf (exp-text-resources exp-win)
          text) ))                      ; text-cell-resource

(defmethod present ((stimulus dot-pair-stimulus)
                     (exp-win experiment-window) )
  (let ((dot-pair (exp-graph-resources exp-win))
        (X1 (stimulus-X1 stimulus))
        (X2 (stimulus-X2 stimulus))
        (Y  (stimulus-Y  stimulus)) )
    (setf (shape-anchor-point (first dot-pair))   ; left dot
          (make-point X1 Y))
    (setf (shape-anchor-point (second dot-pair))  ; right dot
          (make-point X2 Y))
    (setf (exp-visible-stimuli exp-win)           ; display the two dots
          dot-pair) ))

(defmethod present ((feedback-obj text-stimulus)
                     (exp-win experiment-window) )
  (let ((text-cell (exp-text-resources exp-win)))
    (setf (text-shape-string text-cell)
          (text-stimulus-string feedback-obj))
    (setf (shape-anchor-point text-cell)
          (text-stimulus-point feedback-obj))
    (setf (exp-visible-stimuli exp-win)           ; display the string
          (list text-cell)) ))


;;;;;;;;;;;;;;;  Running trials ;;;;;;;;;;;;
;;

(defparameter *beep-duration*        500)  ; milliseconds
(defparameter *feedback-duration*   1000)

(defvar  *experiment-starting-time*  0000000
  "The absolute time of the beginning of the experiment." )
  ;; Subtracted from (get-internal-real-time) to produce the time-stamp of each trial.

(defun code-answer (char)
  "The scale is from 1 to 7. For all chars other than #\1-#\7 return 0."
  (let ((code (- (char-code char) (char-code #\0))))
    (if (<= 1 code 7)
        code
        0 )))

;; Trial schedule -- each trial consists of the following event sequence:
;;   1. Beep -- "Get ready!" signal, blank screen, 500 msec.
;;   2. Stimulus presentation -- the dots are presented on the screen
;;        after the end of the beep and stay there until a key is pressed.
;;        (Note that unlike previous expmts, there is no deadline.)
;;   3. Feedback -- the stimulus is cleared from the screen and a big digit (on
;;        feedback trials) or the letter X (on no-feedback trials) appears and
;;        stays there for *FEEDBACK-DURATION*;
;;   4. Move on to the next trial; its beep interval serves as an ITI.
;;
;; Note that unlike previous experiments, no attempt is being made to control
;; the total trial duration.

(defmethod do-trial ((trial dot-pair-trial)
                      (exp-win experiment-window))
  (let ((start-time (get-internal-real-time))
        (feedback-obj nil) )  ; to be set below
    (beep)
    (clear-window exp-win)
    (setq feedback-obj                         ; do some useful work during the beep
          (corresp-feedback-object (dot-trial-feedback trial)) )
    (setf (text-stimulus-point feedback-obj)   ; the feedback will appear
          (feedback-position trial) )          ; in the position of the stimulus
    (setf (dot-trial-time-stamp trial)
          (- (get-internal-real-time) *experiment-starting-time*))
    (idle-wait (+ start-time *beep-duration*) :deadline)
    ;; The beep is over now; it's time to present the stimuli and wait for a keypress.
    (present trial exp-win)         ; paint the two dots on the screen
    (reset-RT-meter exp-win)        ; start listening to the keyboard
    (multiple-value-bind (char RT)
                         (wait-for-key-press exp-win)   ; no deadline
      ;; A key has beed pressed -- clear the dots and present the feedback
      (present feedback-obj exp-win)     
      (let ((end-time (+ (get-internal-real-time) *feedback-duration*))
             (rating  (code-answer char)) )
        (setf (trial-result trial)  ; do some useful work during the feedback period
              (list rating RT))     ; register data
      (idle-wait end-time :deadline)
      (plusp rating) ))))           ; return NIL on bad keys


;;;;;;;;;;;;;;;  Reporting trials in the data file ;;;;;;;;;;;;
;;

(defvar *group-of-current-subject* 0 )   ; See MAKE-EXPERIMENT at the end of this file.

(defmethod report-trial ((trial dot-pair-trial) (data-file stream))
  (format data-file " ~D ~8D ~3D ~3D  ~3@D ~{~3D ~} ~3D   ~D ~{~6D ~}"
          *group-of-current-subject*
          (dot-trial-time-stamp trial)    ; absolute time [msec] since beg. of expmt
          (second (trial-numbers trial))  ; sub-block-number
          (third  (trial-numbers trial))  ; experiment-trial-number
          (dot-trial-schedule trial)      ; see schedule->code
          (stimulus-code trial)           ; (distance category X1 X2 Y)
          (dot-trial-feedback trial)      ; feedback ; 0="X"
          (trial-status-code (trial-status trial))   ; 0=fail, 1=completed, ...
          (trial-result trial) ))         ; (answer RT)



;;;;;;;;;;;;;;;  Making a trial-pool
;;

(defparameter *number-of-trials-per-mini-block* 28)   ; 1+2+...+7
(defparameter *number-of-mini-blocks-per-block*  4)
(defparameter *number-of-initial-trials*        28)  
;; Note: coordinate with *SUB-BLOCK-SIZES* below

;;;; Sample stimulus description
;;(defparameter *3-categ-1-stim-each-200-pix-apart*
;;  ;; {(category-member-list category-label)}+
;;  '(((275) 1)
;;    ((475) 2)
;;    ((675) 3) ))

(defun  stim-descriptor->dot-trial (stimulus-descriptor feedback)
  (let ((member-list (first stimulus-descriptor))
        (category-label (second stimulus-descriptor)) )
    (make-dot-pair-trial (first member-list)    ; always use first value
                         category-label
                         feedback) ))

(defun make-uniform-pool (vector &optional (feedback 0)
                                           (stimuli *7-categ-1-stim-each-40-pix-apart*))
  ;; The length of VECTOR defines the size of the pool
  (declare (type list stimuli)
           (type vector vector) )
  (let ((circular-list (copy-list stimuli)))
    (setf (cdr (last circular-list)) circular-list)  ;; WARNING!!!
    (flet ((next ()
             (prog1 
               (first circular-list)
               (setf circular-list (rest circular-list))) ))
      (dotimes (i (length vector))
        (setf (svref vector i)
              (stim-descriptor->dot-trial (next) feedback)))
      vector)))

(defun make-left-triangular-pool (vector &optional (feedback 0)
                                          (stimuli *7-categ-1-stim-each-40-pix-apart*))
  ;; relative presentation frequencies:  [1 2 3 ... n]   ; negative skew
  ;; (mapcon #'copy-list '(1 2 3 4))  ==>  (1 2 3 4  2 3 4  3 4  4)
  (make-uniform-pool vector feedback
                     (mapcon #'copy-list stimuli)))  ; unfolded triangle

(defun make-right-triangular-pool (vector &optional (feedback 0)
                                           (stimuli *7-categ-1-stim-each-40-pix-apart*))
  ;; relative presentation frequencies:  [n n-1 n-2 ... 1]  ; positive skew
  (make-left-triangular-pool vector feedback (reverse stimuli)))


(defun make-mini-trial-pool (schedule &key
                                       (stimuli *7-categ-1-stim-each-40-pix-apart*)
                                       (size *number-of-trials-per-mini-block*) )
  (let ((vector (make-array size :element-type 'dot-pair-trial))
        (schedule-code  (schedule->code schedule)) )
    (case schedule-code
      (0  (make-uniform-pool          vector :category stimuli))    ; initial, feedback
      (1  (make-uniform-pool          vector :category stimuli))    ; U with feedback
      (2  (make-uniform-pool          vector 0         stimuli))    ; U w/o  feedback
      (3  (make-right-triangular-pool vector :category stimuli))    ; P with feedback
      (4  (make-right-triangular-pool vector 0         stimuli))    ; P w/o  feedback
      (5  (make-left-triangular-pool  vector :category stimuli))    ; N with feedback
      (6  (make-left-triangular-pool  vector 0         stimuli)) )  ; N w/o  feedback
    (dotimes (i size) 
      (setf (dot-trial-schedule (svref vector i)) schedule-code))
    (randomize-trial-pool (make-trial-pool vector size   ; wrap VECTOR in a trial-pool obj
                                           :randomize-p nil)) ))

(defun make-dot-trial-pool (schedule &key
                                       (stimuli *7-categ-1-stim-each-40-pix-apart*)
                                       (mini-size *number-of-trials-per-mini-block*)
                                       (mini-blocks *number-of-mini-blocks-per-block*) )
  ;; Stack the specified number of mini-blocks together, all with the same schedule.
  (do ((k 1 (+ k 1))
       (pool (make-mini-trial-pool schedule :stimuli stimuli :size mini-size)
             (merge-trial-pools pool
                    (make-mini-trial-pool schedule :stimuli stimuli :size mini-size))) )
      ((>= k mini-blocks) pool) ))
             

;;;; Debugging scaffold
#|
(defun print-trial-descriptors (trial-pool &optional (stream *standard-output*))
  (let ((vector (trial-pool trial-pool))
        (size (trial-pool-size trial-pool)) )
    (dotimes (i size)
      (format stream "~%~3D   " i)
      (report-trial (aref vector i) stream))
    size ))
|#


;;;;;;;;;;;;;;;  Building the experiment ;;;;;;;;;;;;
;;
;;  This section creates the experiment by constructing its various
;;  blocks and sub-blocks.  Conceptually, there are five blocks as follows:
;;   + block1 -- Short warm-up segment (:init).
;;   + block2 -- First feedback block (:U-fb, :P-fb, or :N-fb).
;;   + block3 -- First no-feedback block (:U-no, :P-no, or :N-no, respectively).
;;   + block4 -- Second feedback block (same schedule as block2).
;;   + block5 -- Second no-feedback block (same schedule as block3).
;;
;;  These five 'conceptual' blocks are concatenated into a single long BLOCK object.
;;  See the function MAKE-BIG-BLOCK below.
;;
;;  The subject needs breaks. This is implemented by sub-blocks. Their only purpose is to
;;  provide breaks by means of finish-instruction dialog windows.  There are 8 sub-blocks
;;  of various duration. The transitions between the 'conceptual' blocks (except the 
;;  warm-up) are marked by break periods.

(defparameter *sub-block-sizes*
  '(70 70  56 56  56 56  56 :all) )   ; total 476 = 28 + 4*112


;;  The subjects are assigned to three groups numbered 1-3.
;;
(defun  group->exp-schedule (group-number)
  (ecase group-number
    (1 '(:init :U-fb :U-no :U-fb :U-no))      ; (0 1 2 1 2)  ; Group U
    (2 '(:init :P-fb :P-no :P-fb :P-no))      ; (0 3 4 3 4)  ; Group P
    (3 '(:init :N-fb :N-no :N-fb :N-no))      ; (0 5 6 5 6)  ; Group N
    (4 '(:init :U-no :U-fb :U-no :U-fb))      ; (0 2 1 2 1)  ; Group U1
    (5 '(:init :P-no :P-fb :P-no :P-fb))      ; (0 4 3 4 3)  ; Group P1
    (6 '(:init :N-no :N-fb :N-no :N-fb)) ))   ; (0 6 5 6 5)  ; Group N1


;;;;;;;;;;;;;;;  Instructions ;;;;;;;;;;;;
;;

(defparameter *exp-start-instruct*  '(
"During this experiment you will see pairs of white dots on a black screen.
One such pair is presented on each trial. Your task is to label each pair by a number
from 1 to 7 indicating the distance between the two dots. There are only 7 stimuli 
(that is, distances) that will be shown to you multiple times in random order.
Seven stimuli, seven responses. The greater the distance, the greater the response.

On all trials the two dots form a horizontal segment; there are no oblique segments.
The exact position of the segment varies from trial to trial -- the segment will appear
to move up and down as well as left and right.  Your task is to identify the LENGTH of 
this segment regardless of its location on the screen.

Approximately half the trials provide feedback -- a big digit appears after you have
entered your response. The digit indicates the correct answer.  There are also trials
without feedback in which the letter X appears on the screen instead. You should never
type X as your response on any trial; valid responses are only the digits from 1 to 7.
Try to answer as consistently as you can.

Please enter your ratings on the numeric keypad at the right part of the keyboard.
The speed of your responses is being recorded, so please try to respond as quickly as 
possible without being careless.


Press RETURN to see the next page of instructions."

  ;; ------- new page

"The experiment lasts about 40 minutes and consists of 450 trials organized in 8 blocks.  
There are short breaks between the blocks.  Feel free to take time to rest during these
breaks.  This will improve your concentration and hence your performance.

The structure of each trial is the following:
   A brief beep signals the beginning of the trial.  The two dots appear immediately
   after the beep and stay on the screen until you press a number from 1 to 7.
   When you enter your response, the dots disapear and a digit indicating the correct
   response (or the letter X) appears on feedback trials. It stays on for one second,
   after which the screen is cleared and you move to the next trial.


   *************************************************************
   ***  Please try to respond as quickly as possible without being careless.  ***
   *************************************************************


If you have any questions about the procedure, please ask the experimenter now.
Otherwise press RETURN to start with the first trial."
))  ; *exp-start-instruct*


(defparameter *exp-finish-instruct*  '(
"The experiment is over.

Thank you for your cooperation.  Your patience and concentration are highly appreciated.

Please see the experimenter.  He will be glad to explain the experiment in more detail
and will give you a debriefing sheet."
)) ; *exp-finish-instruct*


(defparameter *block-start-instruct*  nil )
(defparameter *block-finish-instruct* nil )


;;;;;;;;;;;;;;;  Make the single big experimental block ;;;;;;;;;;;;
;;

(defun make-big-trial-pool (exp-schedule)
  "Makes a single big pool by merging five smaller pools together."
  ;; Uses the default values defined by various global variables.
  ;; Assumes the first element of EXP-SCHEDULE is always :INIT
  (let* ((pool1 (make-mini-trial-pool :init :size *number-of-initial-trials*))
         (pool2 (make-dot-trial-pool (second  exp-schedule)))
         (pool3 (make-dot-trial-pool (third   exp-schedule)))
         (pool4 (make-dot-trial-pool (fourth  exp-schedule)))
         (pool5 (make-dot-trial-pool (fifth   exp-schedule)))
         (big-pool (merge-trial-pools pool5 pool4 pool3 pool2 pool1)) )
         ;; Stack the pools in reverse order because NEXT-TRIAL goes bckwds (see trial.lisp)
    (setf (trial-pool-randomize-p big-pool) nil)
    (setf (trial-pool-repeat-on-fail-p big-pool) nil)
    big-pool ))


(defun make-big-block (exp-schedule &key (sub-block-sizes  *sub-block-sizes*)
                                           (start-instruct   *block-start-instruct*)
                                           (finish-instruct  *block-finish-instruct*) )
  (let ((trial-pool (make-big-trial-pool exp-schedule))  ; use default STIMULI, etc
        (sub-block-list (mapcar #'make-sub-block sub-block-sizes)) )   ; see exp_win.lisp
    (make-chain-finish-instructions sub-block-list)
    (make-block trial-pool 
                sub-block-list
                :start-instruct  start-instruct
                :finish-instruct finish-instruct) ))


(defun make-chain-finish-instructions (sub-block-list)
  "Destructively set the finish-instruct slot of each sub-block."
  (let ((number-of-sub-blocks (length sub-block-list)))
    (dotimes (i (- number-of-sub-blocks 1))
      (push (format nil "Finished block ~D of ~D of the experiment.~%~@
                         Please take a small break to improve concentration.~%~@
                         Press RETURN to continue with the next block."
                    (+ i 1)  number-of-sub-blocks)
            (finish-instruct (nth i sub-block-list)))) ))


;;;;;;;;  Finally, put it all together
;;

(defun make-experiment (subject-number group &rest keyargs)
  (declare (ignore keyargs))  ; for the moment
  (let ((block (make-big-block (group->exp-schedule group)))
        (win   (make-instance 'experiment-window)) )
    (setq *group-of-current-subject* group)
    (setup-experiment win
                      (list block)
                      subject-number
                      :background-color *background-color*
                      :foreground-color *foreground-color*
                      :start-instruct  *exp-start-instruct*
                      :finish-instruct *exp-finish-instruct*)
    ;; Set font size to 200
    (let ((font-descriptor (view-font win)))
      (setf (second font-descriptor) 200)
      (set-view-font win font-descriptor) )
    win ))


;; The primary method for run-experiment is defined in Experimts:Exp_win.lisp.

(defmethod run-experiment :before ((exp-win experiment-window))
  (prepare-presentation (make-dot-pair-trial 300 1 1) 
                        exp-win)
  (setq *experiment-starting-time* (get-internal-real-time)) )


;;;;;;;  End of file EXPERMTS/MILLER1/MILLER1_EXP1.LISP
